import json
from unittest.mock import MagicMock

from pypanther import LogType, Rule, RuleMock, RuleTest, Severity, panther_managed
from pypanther.helpers.github import github_alert_context


@panther_managed
class GitHubActionFailed(Rule):
    id = "GitHub.Action.Failed-prototype"
    display_name = "GitHub Action Failed"
    enabled = False
    log_types = [LogType.GITHUB_AUDIT]
    tags = ["GitHub", "Configuration Required"]
    default_severity = Severity.HIGH
    default_description = "A monitored github action has failed."
    default_runbook = "Inspect the action failure link and take appropriate response. There are no general plans of response for this activity.\n"
    default_reference = (
        "https://docs.github.com/en/actions/creating-actions/setting-exit-codes-for-actions#about-exit-codes"
    )
    # The keys for MONITORED_ACTIONS are gh_org/repo_name
    # The values for MONITORED_ACTIONS are a list of ["action_names"]
    MONITORED_ACTIONS = {}

    def rule(self, event):
        if isinstance(self.MONITORED_ACTIONS, MagicMock):
            self.MONITORED_ACTIONS = json.loads(self.MONITORED_ACTIONS())  # pylint: disable=not-callable
        repo = event.get("repo", "")
        action_name = event.get("name", "")
        return all(
            [
                event.get("action", "") == "workflows.completed_workflow_run",
                event.get("conclusion", "") == "failure",
                repo in self.MONITORED_ACTIONS,
                action_name in self.MONITORED_ACTIONS.get(repo, []),
            ],
        )

    def title(self, event):
        repo = event.get("repo", "<NO_REPO>")
        action_name = event.get("name", "<NO_ACTION_NAME>")
        return f"GitHub Action [{action_name}] in [{repo}] has failed"

    def alert_context(self, event):
        a_c = github_alert_context(event)
        a_c["action"] = event.get("name", "<NO_ACTION_NAME>")
        a_c["action_run_link"] = (
            f"https://github.com/{a_c.get('repo')}/actions/runs/{event.get('workflow_run_id', '<NO_RUN_ID>')}"
        )
        return a_c

    tests = [
        RuleTest(
            name="GitHub - Branch Protection Disabled",
            expected_result=False,
            log={
                "actor": "cat",
                "action": "protected_branch.destroy",
                "created_at": 1621305118553,
                "org": "my-org",
                "p_log_type": "GitHub.Audit",
                "repo": "my-org/my-repo",
            },
        ),
        RuleTest(
            name="GitHub Action Failed - No Configuration",
            expected_result=False,
            log={
                "_document_id": "pWWWWWWWWWWWWWWWWWWWWW",
                "action": "workflows.completed_workflow_run",
                "actor": "github_handle",
                "at_sign_timestamp": "2023-01-31 18:58:27.638",
                "business": "github-business-only-if-enterprise-audit-log",
                "completed_at": "2023-01-31T18:58:27.000Z",
                "conclusion": "failure",
                "created_at": "2023-01-31 18:58:27.638",
                "event": "schedule",
                "head_branch": "master",
                "head_sha": "66dddddddddddddddddddddddddddddddddddddd",
                "name": "sync-panther-analysis-from-upstream",
                "operation_type": "modify",
                "org": "panther-labs",
                "p_log_type": "GitHub.Audit",
                "public_repo": False,
                "repo": "your-org/panther-analysis-copy",
                "run_attempt": 3,
                "run_number": 99,
                "started_at": "2023-01-31 18:58:04",
                "workflow_id": 44444444,
                "workflow_run_id": 5555555555,
            },
        ),
        RuleTest(
            name="GitHub Action Failed - Monitored Action Configured",
            expected_result=True,
            mocks=[
                RuleMock(
                    object_name="MONITORED_ACTIONS",
                    return_value='{\n  "your-org/panther-analysis-copy": [ "sync-panther-analysis-from-upstream"]\n}',
                ),
            ],
            log={
                "_document_id": "pWWWWWWWWWWWWWWWWWWWWW",
                "action": "workflows.completed_workflow_run",
                "actor": "github_handle",
                "at_sign_timestamp": "2023-01-31 18:58:27.638",
                "business": "github-business-only-if-enterprise-audit-log",
                "completed_at": "2023-01-31T18:58:27.000Z",
                "conclusion": "failure",
                "created_at": "2023-01-31 18:58:27.638",
                "event": "schedule",
                "head_branch": "master",
                "head_sha": "66dddddddddddddddddddddddddddddddddddddd",
                "name": "sync-panther-analysis-from-upstream",
                "operation_type": "modify",
                "org": "panther-labs",
                "p_log_type": "GitHub.Audit",
                "public_repo": False,
                "repo": "your-org/panther-analysis-copy",
                "run_attempt": 3,
                "run_number": 99,
                "started_at": "2023-01-31 18:58:04",
                "workflow_id": 44444444,
                "workflow_run_id": 5555555555,
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_rule_context


@panther_managed
class AWSUnsuccessfulMFAattempt(Rule):
    default_description = "Monitor application logs for suspicious events including repeated MFA failures that may indicate user's primary credentials have been compromised."
    display_name = "AWS Unsuccessful MFA attempt"
    enabled = False
    default_reference = "https://attack.mitre.org/techniques/T1621/"
    tags = ["Configuration Required"]
    reports = {"MITRE ATT&CK": ["TA0006:T1621"]}
    default_severity = Severity.HIGH
    dedup_period_minutes = 15
    log_types = [LogType.AWS_CLOUDTRAIL]
    id = "AWS.Unsuccessful.MFA.attempt-prototype"
    threshold = 2

    def rule(self, event):
        if event.get("eventSource") != "signin.amazonaws.com" and event.get("eventName") != "ConsoleLogin":
            return False
        mfa_used = event.deep_get("additionalEventData", "MFAUsed", default="")
        console_login = event.deep_get("responseElements", "ConsoleLogin", default="")
        if mfa_used == "Yes" and console_login == "Failure":
            return True
        return False

    def title(self, event):
        arn = event.deep_get("userIdenity", "arn", default="No ARN")
        username = event.deep_get("userIdentity", "userName", default="No Username")
        return f"Failed MFA login from [{arn}] [{username}]"

    def alert_context(self, event):
        return aws_rule_context(event)

    tests = [
        RuleTest(
            name="Successful Login w/ MFA",
            expected_result=False,
            log={
                "additionalEventData": {
                    "LoginTo": "https://console.aws.amazon.com/console/home?state=hashArgs%23&isauthcode=true",
                    "MFAIdentifier": "arn:aws:iam::111122223333:u2f/user/anaya/default-AAAAAAAABBBBBBBBCCCCCCCCDD",
                    "MFAUsed": "Yes",
                    "MobileVersion": "No",
                },
                "awsRegion": "us-east-1",
                "eventID": "fed06f42-cb12-4764-8c69-121063dc79b9",
                "eventName": "ConsoleLogin",
                "eventSource": "signin.amazonaws.com",
                "eventTime": "2022-11-10T16:24:34Z",
                "eventType": "AwsConsoleSignIn",
                "eventVersion": "1.05",
                "recipientAccountId": "111122223333",
                "requestParameters": None,
                "responseElements": {"ConsoleLogin": "Success"},
                "sourceIPAddress": "192.0.2.0",
                "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/67.0.3396.99 Safari/537.36",
                "userIdentity": {
                    "accountId": "111122223333",
                    "arn": "arn:aws:iam::111122223333:user/anaya",
                    "principalId": "AIDACKCEVSQ6C2EXAMPLE",
                    "type": "IAMUser",
                    "userName": "anaya",
                },
            },
        ),
        RuleTest(
            name="Unsuccessful Login w/ MFA",
            expected_result=True,
            log={
                "additionalEventData": {
                    "LoginTo": "https://console.aws.amazon.com/console/home?state=hashArgs%23&isauthcode=true",
                    "MFAUsed": "Yes",
                    "MobileVersion": "No",
                },
                "awsRegion": "us-east-1",
                "errorMessage": "Failed authentication",
                "eventID": "d38ce1b3-4575-4cb8-a632-611b8243bfc3",
                "eventName": "ConsoleLogin",
                "eventSource": "signin.amazonaws.com",
                "eventTime": "2022-11-10T16:24:34Z",
                "eventType": "AwsConsoleSignIn",
                "eventVersion": "1.05",
                "recipientAccountId": "111122223333",
                "requestParameters": None,
                "responseElements": {"ConsoleLogin": "Failure"},
                "sourceIPAddress": "192.0.2.0",
                "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/67.0.3396.99 Safari/537.36",
                "userIdentity": {
                    "accessKeyId": "",
                    "accountId": "111122223333",
                    "principalId": "AIDACKCEVSQ6C2EXAMPLE",
                    "type": "IAMUser",
                    "userName": "anaya",
                },
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class AsanaTeamPrivacyPublic(Rule):
    default_description = (
        "An Asana team's privacy setting was changed to public to the organization (not public to internet)"
    )
    display_name = "Asana Team Privacy Public"
    default_reference = "https://help.asana.com/hc/en-us/articles/14211433439387-Team-permissions"
    default_severity = Severity.LOW
    log_types = [LogType.ASANA_AUDIT]
    id = "Asana.Team.Privacy.Public-prototype"

    def rule(self, event):
        return (
            event.get("event_type") == "team_privacy_settings_changed"
            and event.deep_get("details", "new_value") == "public"
        )

    def title(self, event):
        team = event.deep_get("resource", "name", default="<TEAM_NOT_FOUND>")
        actor = event.deep_get("actor", "email", default="<ACTOR_NOT_FOUND>")
        return f"Asana team [{team}] has been made public to the org by [{actor}]."

    tests = [
        RuleTest(
            name="Team made public",
            expected_result=True,
            log={
                "actor": {
                    "actor_type": "user",
                    "email": "homer.simpson@panther.io",
                    "gid": "12345",
                    "name": "Homer Simpson",
                },
                "context": {
                    "client_ip_address": "12.12.12.12",
                    "context_type": "web",
                    "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
                },
                "created_at": "2022-12-16 19:35:21.026",
                "details": {"new_value": "public"},
                "event_category": "access_control",
                "event_type": "team_privacy_settings_changed",
                "gid": "12345",
                "resource": {"gid": "12345", "name": "Example Team Name", "resource_type": "team"},
                "p_log_type": "Asana.Audit",
            },
        ),
        RuleTest(
            name="Other event",
            expected_result=False,
            log={
                "actor": {
                    "actor_type": "user",
                    "email": "homer.simpsons@simpsons.com",
                    "gid": "1234567890",
                    "name": "Homer Simpson",
                },
                "context": {
                    "client_ip_address": "1.2.3.4",
                    "context_type": "web",
                    "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/109.0.0.0 Safari/537.36",
                },
                "created_at": "2023-02-01 18:05:08.413",
                "details": {"method": ["SAML"]},
                "event_category": "logins",
                "event_type": "user_login_succeeded",
                "gid": "123456789",
                "resource": {
                    "email": "homer.simpsons@simpsons.com",
                    "gid": "1234567890",
                    "name": "Homer Simpson",
                    "resource_type": "user",
                },
                "p_log_type": "Asana.Audit",
            },
        ),
    ]



import pycountry

from pypanther import LogType, Rule, RuleMock, RuleTest, Severity, panther_managed
from pypanther.helpers import event_type


@panther_managed
class StandardSignInFromRogueState(Rule):
    id = "Standard.SignInFromRogueState-prototype"
    display_name = "Sign In from Rogue State"
    log_types = [
        LogType.ASANA_AUDIT,
        LogType.ATLASSIAN_AUDIT,
        LogType.AWS_CLOUDTRAIL,
        LogType.AZURE_AUDIT,
        LogType.BOX_EVENT,
        LogType.NOTION_AUDIT_LOGS,
        LogType.OKTA_SYSTEM_LOG,
        LogType.ONELOGIN_EVENTS,
        LogType.ONEPASSWORD_SIGN_IN_ATTEMPT,
        LogType.ZENDESK_AUDIT,
        LogType.ZOOM_ACTIVITY,
    ]
    default_severity = Severity.MEDIUM
    reports = {"MITRE ATT&CK": ["TA0001:T1078.004"]}
    default_description = "Detects when an entity signs in from a nation associated with cyber attacks"
    tags = ["DataModel", "Configuration Required"]
    # Configuration Required:
    #   Configure the below list of rogue states according to your needs/experience
    #   Refer to the link below to find the alpha-2 code corresponding to your country
    #   https://www.iban.com/country-codes
    ROGUE_STATES = {"CN", "IR", "RU"}

    def rule(self, event):
        # Only evaluate successful logins
        if event.udm("event_type") != event_type.SUCCESSFUL_LOGIN:
            return False
        # Ignore events with no IP data
        if not event.udm("source_ip"):
            return False
        # Get contry of request origin and compare to identified rogue state list
        country = self.get_country(event)
        if country is None:
            # We weren't able to find a matching country, therefore we don't have enough information
            #   to alert on
            return False
        #   Wrapping in 'bool' so that we can use mocking for 'is_rogue_state'
        return bool(self.is_rogue_state(country.alpha_2))

    def title(self, event):
        log_type = event.get("p_log_type")
        country = self.get_country(event)
        account_name = self.get_account_name(event)
        return f"{log_type}: Sign-In for account {account_name} from Rogue State '{country.name}'"

    def alert_context(self, event):
        return {
            "source_ip": event.udm("source_ip"),
            "country": self.get_country(event).name,
            "account_name": self.get_account_name(event),
        }

    def get_country(self, event) -> str:
        """Returns the country code from an event's IPinfo data."""
        location_data = event.deep_get("p_enrichment", "ipinfo_location", event.udm_path("source_ip"))
        if not location_data:
            return None  # Ignore event if we have no enrichment to analyze
        return pycountry.countries.get(alpha_2=location_data.get("country").upper())

    def get_account_name(self, event) -> str:
        """Returns the account name."""
        if account_name := event.udm("actor_user"):
            return account_name
        return "UNKNWON ACCOUNT"

    def is_rogue_state(self, country_code: str) -> bool:
        """Returns whether the country code provided belongs to an identified rogue state."""
        # This function makes it easy for us to use unit test mocks to ensure altering the ROGUE_STATES
        #   dict doesn't break our test suite.
        return country_code in self.ROGUE_STATES

    tests = [
        RuleTest(
            name="Non-Sign-In Event",
            expected_result=False,
            log={
                "actor": {
                    "alternateId": "dude.lightbulb@example.co",
                    "displayName": "Dude Lightbulb",
                    "id": "EXAMPLE_ACTOR_ID",
                    "type": "User",
                },
                "authenticationContext": {"authenticationStep": 0, "externalSessionId": "EXAMPLE_SESSION_ID"},
                "client": {
                    "device": "Computer",
                    "geographicalContext": {
                        "city": "Winnipeg",
                        "country": "Canada",
                        "geolocation": {"lat": 49.922, "lon": -96.965},
                        "postalCode": "R2C",
                        "state": "Manitoba",
                    },
                    "ipAddress": "1.1.1.1",
                    "userAgent": {
                        "browser": "CHROME",
                        "os": "Mac OS X",
                        "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/127.0.0.0 Safari/537.36",
                    },
                    "zone": "null",
                },
                "displayMessage": "User single sign on to app",
                "eventType": "user.authentication.sso",
                "legacyEventType": "app.auth.sso",
                "outcome": {"result": "SUCCESS"},
                "p_any_ip_addresses": ["1.1.1.1"],
                "p_enrichment": {
                    "ipinfo_location": {
                        "client.ipAddress": {
                            "city": "Winnipeg",
                            "country": "CA",
                            "lat": "49.8844",
                            "lng": "-97.14704",
                            "p_match": "1.1.1.1",
                            "postal_code": "R3B",
                            "region": "Manitoba",
                            "region_code": "MB",
                            "timezone": "America/Winnipeg",
                        },
                    },
                },
                "p_log_type": "Okta.SystemLog",
                "request": {
                    "ipChain": [
                        {
                            "geographicalContext": {
                                "city": "Winnipeg",
                                "country": "Canada",
                                "geolocation": {"lat": 49.922, "lon": -96.965},
                                "postalCode": "R2C",
                                "state": "Manitoba",
                            },
                            "ip": "1.1.1.1",
                            "version": "V4",
                        },
                    ],
                },
                "severity": "INFO",
                "transaction": {"detail": {}, "id": "a33d5f8d1669b80efb7338791e222908", "type": "WEB"},
                "uuid": "6270d421-5be3-11ef-9376-abec352bd6d0",
                "version": "0",
            },
        ),
        RuleTest(
            name="Sign-In From Inconspicuous State",
            expected_result=False,
            log={
                "actor": {
                    "alternateId": "dude.lightbulb@example.co",
                    "displayName": "Dude Lightbulb",
                    "id": "EXAMPLE_USER_ID",
                    "type": "User",
                },
                "authenticationContext": {"authenticationStep": 0, "externalSessionId": "EXAMPLE_SESSION_ID"},
                "client": {
                    "device": "Computer",
                    "geographicalContext": {
                        "city": "Winnipeg",
                        "country": "Canada",
                        "geolocation": {"lat": 49.922, "lon": -96.965},
                        "postalCode": "R2C",
                        "state": "Manitoba",
                    },
                    "ipAddress": "1.1.1.1",
                    "userAgent": {
                        "browser": "CHROME",
                        "os": "Mac OS X",
                        "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36",
                    },
                    "zone": "null",
                },
                "debugContext": {
                    "debugData": {
                        "authnRequestId": "xxx",
                        "deviceFingerprint": "xxx",
                        "dtHash": "xxx",
                        "logOnlySecurityData": '{"risk":{"level":"LOW"},"behaviors":{"New Geo-Location":"NEGATIVE","New Device":"NEGATIVE","New IP":"NEGATIVE","New State":"NEGATIVE","New Country":"NEGATIVE","Velocity":"NEGATIVE","New City":"NEGATIVE"}}',
                        "oktaUserAgentExtended": "okta-auth-js/7.7.0 okta-signin-widget-7.21.0",
                        "origin": "https://example.okta.com",
                        "requestId": "xxx",
                        "requestUri": "/idp/idx/identify",
                        "threatSuspected": "false",
                        "url": "/idp/idx/identify?",
                    },
                },
                "displayMessage": "User login to Okta",
                "eventType": "user.session.start",
                "legacyEventType": "core.user_auth.login_success",
                "outcome": {"result": "SUCCESS"},
                "p_enrichment": {
                    "ipinfo_location": {
                        "client.ipAddress": {
                            "city": "Winnipeg",
                            "country": "CA",
                            "lat": "49.8844",
                            "lng": "-97.14704",
                            "p_match": "1.1.1.1",
                            "postal_code": "R3B",
                            "region": "Manitoba",
                            "region_code": "MB",
                            "timezone": "America/Winnipeg",
                        },
                    },
                },
                "p_event_time": "2024-08-15 15:05:09.154000000",
                "p_log_type": "Okta.SystemLog",
                "p_parse_time": "2024-08-15 15:08:22.176160519",
                "p_row_id": "526383f6c367e7f3db88959621afee19",
                "p_source_id": "d0907120-58a3-4e40-acfa-e631693f9066",
                "p_source_label": "My Log Source",
                "published": "2024-08-15 15:05:09.154000000",
                "request": {
                    "ipChain": [
                        {
                            "geographicalContext": {
                                "city": "Winnipeg",
                                "country": "Canada",
                                "geolocation": {"lat": 49.922, "lon": -96.965},
                                "postalCode": "R2C",
                                "state": "Manitoba",
                            },
                            "ip": "1.1.1.1",
                            "version": "V4",
                        },
                    ],
                },
                "securityContext": {
                    "asNumber": 7122,
                    "asOrg": "SAMPLE_ISP",
                    "domain": "isp.net",
                    "isProxy": False,
                    "isp": "SAMPLE_ISP",
                },
                "severity": "INFO",
                "target": [
                    {"alternateId": "My Okta App", "displayName": "My Okta App", "id": "xxx", "type": "AppInstance"},
                ],
                "transaction": {"detail": {}, "id": "32caf8cb5819a0928702b4b835e163a0", "type": "WEB"},
                "uuid": "c35900e7-5b17-11ef-ad6d-cf78a9534d8d",
                "version": "0",
            },
        ),
        RuleTest(
            name="Sign-In with no Enrichment Data",
            expected_result=False,
            log={
                "actor": {
                    "alternateId": "dude.lightbulb@example.co",
                    "displayName": "Dude Lightbulb",
                    "id": "EXAMPLE_USER_ID",
                    "type": "User",
                },
                "authenticationContext": {"authenticationStep": 0, "externalSessionId": "EXAMPLE_SESSION_ID"},
                "client": {
                    "device": "Computer",
                    "geographicalContext": {
                        "city": "Winnipeg",
                        "country": "Canada",
                        "geolocation": {"lat": 49.922, "lon": -96.965},
                        "postalCode": "R2C",
                        "state": "Manitoba",
                    },
                    "ipAddress": "1.1.1.1",
                    "userAgent": {
                        "browser": "CHROME",
                        "os": "Mac OS X",
                        "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36",
                    },
                    "zone": "null",
                },
                "debugContext": {
                    "debugData": {
                        "authnRequestId": "xxx",
                        "deviceFingerprint": "xxx",
                        "dtHash": "xxx",
                        "logOnlySecurityData": '{"risk":{"level":"LOW"},"behaviors":{"New Geo-Location":"NEGATIVE","New Device":"NEGATIVE","New IP":"NEGATIVE","New State":"NEGATIVE","New Country":"NEGATIVE","Velocity":"NEGATIVE","New City":"NEGATIVE"}}',
                        "oktaUserAgentExtended": "okta-auth-js/7.7.0 okta-signin-widget-7.21.0",
                        "origin": "https://example.okta.com",
                        "requestId": "xxx",
                        "requestUri": "/idp/idx/identify",
                        "threatSuspected": "false",
                        "url": "/idp/idx/identify?",
                    },
                },
                "displayMessage": "User login to Okta",
                "eventType": "user.session.start",
                "legacyEventType": "core.user_auth.login_success",
                "outcome": {"result": "SUCCESS"},
                "p_event_time": "2024-08-15 15:05:09.154000000",
                "p_log_type": "Okta.SystemLog",
                "p_parse_time": "2024-08-15 15:08:22.176160519",
                "p_row_id": "526383f6c367e7f3db88959621afee19",
                "p_source_id": "d0907120-58a3-4e40-acfa-e631693f9066",
                "p_source_label": "My Log Source",
                "published": "2024-08-15 15:05:09.154000000",
                "request": {
                    "ipChain": [
                        {
                            "geographicalContext": {
                                "city": "Winnipeg",
                                "country": "Canada",
                                "geolocation": {"lat": 49.922, "lon": -96.965},
                                "postalCode": "R2C",
                                "state": "Manitoba",
                            },
                            "ip": "1.1.1.1",
                            "version": "V4",
                        },
                    ],
                },
                "securityContext": {
                    "asNumber": 7122,
                    "asOrg": "SAMPLE_ISP",
                    "domain": "isp.net",
                    "isProxy": False,
                    "isp": "SAMPLE_ISP",
                },
                "severity": "INFO",
                "target": [
                    {"alternateId": "My Okta App", "displayName": "My Okta App", "id": "xxx", "type": "AppInstance"},
                ],
                "transaction": {"detail": {}, "id": "32caf8cb5819a0928702b4b835e163a0", "type": "WEB"},
                "uuid": "c35900e7-5b17-11ef-ad6d-cf78a9534d8d",
                "version": "0",
            },
        ),
        RuleTest(
            name="Sign-In From Country That Doesn't Exist",
            expected_result=False,
            log={
                "actor": {
                    "alternateId": "dude.lightbulb@example.co",
                    "displayName": "Dude Lightbulb",
                    "id": "EXAMPLE_USER_ID",
                    "type": "User",
                },
                "authenticationContext": {"authenticationStep": 0, "externalSessionId": "EXAMPLE_SESSION_ID"},
                "client": {
                    "device": "Computer",
                    "geographicalContext": {
                        "city": "Winnipeg",
                        "country": "Canada",
                        "geolocation": {"lat": 49.922, "lon": -96.965},
                        "postalCode": "R2C",
                        "state": "Manitoba",
                    },
                    "ipAddress": "1.1.1.1",
                    "userAgent": {
                        "browser": "CHROME",
                        "os": "Mac OS X",
                        "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36",
                    },
                    "zone": "null",
                },
                "debugContext": {
                    "debugData": {
                        "authnRequestId": "xxx",
                        "deviceFingerprint": "xxx",
                        "dtHash": "xxx",
                        "logOnlySecurityData": '{"risk":{"level":"LOW"},"behaviors":{"New Geo-Location":"NEGATIVE","New Device":"NEGATIVE","New IP":"NEGATIVE","New State":"NEGATIVE","New Country":"NEGATIVE","Velocity":"NEGATIVE","New City":"NEGATIVE"}}',
                        "oktaUserAgentExtended": "okta-auth-js/7.7.0 okta-signin-widget-7.21.0",
                        "origin": "https://example.okta.com",
                        "requestId": "xxx",
                        "requestUri": "/idp/idx/identify",
                        "threatSuspected": "false",
                        "url": "/idp/idx/identify?",
                    },
                },
                "displayMessage": "User login to Okta",
                "eventType": "user.session.start",
                "legacyEventType": "core.user_auth.login_success",
                "outcome": {"result": "SUCCESS"},
                "p_enrichment": {
                    "ipinfo_location": {
                        "client.ipAddress": {
                            "city": "Winnipeg",
                            "country": "FAKE_COUNTRY",
                            "lat": "49.8844",
                            "lng": "-97.14704",
                            "p_match": "1.1.1.1",
                            "postal_code": "R3B",
                            "region": "Manitoba",
                            "region_code": "MB",
                            "timezone": "America/Winnipeg",
                        },
                    },
                },
                "p_event_time": "2024-08-15 15:05:09.154000000",
                "p_log_type": "Okta.SystemLog",
                "p_parse_time": "2024-08-15 15:08:22.176160519",
                "p_row_id": "526383f6c367e7f3db88959621afee19",
                "p_source_id": "d0907120-58a3-4e40-acfa-e631693f9066",
                "p_source_label": "My Log Source",
                "published": "2024-08-15 15:05:09.154000000",
                "request": {
                    "ipChain": [
                        {
                            "geographicalContext": {
                                "city": "Winnipeg",
                                "country": "Canada",
                                "geolocation": {"lat": 49.922, "lon": -96.965},
                                "postalCode": "R2C",
                                "state": "Manitoba",
                            },
                            "ip": "1.1.1.1",
                            "version": "V4",
                        },
                    ],
                },
                "securityContext": {
                    "asNumber": 7122,
                    "asOrg": "SAMPLE_ISP",
                    "domain": "isp.net",
                    "isProxy": False,
                    "isp": "SAMPLE_ISP",
                },
                "severity": "INFO",
                "target": [
                    {"alternateId": "My Okta App", "displayName": "My Okta App", "id": "xxx", "type": "AppInstance"},
                ],
                "transaction": {"detail": {}, "id": "32caf8cb5819a0928702b4b835e163a0", "type": "WEB"},
                "uuid": "c35900e7-5b17-11ef-ad6d-cf78a9534d8d",
                "version": "0",
            },
        ),
        RuleTest(
            name="Asana - Rogue State Sign-In",
            expected_result=True,
            mocks=[RuleMock(object_name="is_rogue_state", return_value=True)],
            log={
                "actor": {
                    "actor_type": "user",
                    "email": "dude.lightbulb@example.co",
                    "gid": "xxx",
                    "name": "Dude Lightbulb",
                },
                "context": {
                    "client_ip_address": "1.1.1.1",
                    "context_type": "web",
                    "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/109.0.0.0 Safari/537.36",
                },
                "created_at": "2023-02-16 06:47:34.903",
                "details": {"method": ["SAML"]},
                "event_category": "logins",
                "event_type": "user_login_succeeded",
                "gid": "xxx",
                "p_enrichment": {
                    "ipinfo_location": {
                        "context.client_ip_address": {
                            "city": "Moscow",
                            "country": "RU",
                            "lat": "55.7520",
                            "lng": "37.6150",
                            "p_match": "1.1.1.1",
                            "postal_code": "119021",
                            "region": "Moscow",
                            "region_code": "RU",
                            "timezone": "Europe/Moscow",
                        },
                    },
                },
                "p_event_time": "2023-02-16 06:47:34.903",
                "p_log_type": "Asana.Audit",
                "p_parse_time": "2023-02-16 06:53:22.561",
                "p_row_id": "22bc6744332dc49e86f4a9b816f18a0f",
                "p_schema_version": 0,
                "p_source_id": "46bc875b-172f-4e9b-b475-6efac507c9a2",
                "p_source_label": "Asana",
                "resource": {
                    "email": "dude.lightbulb@example.co",
                    "gid": "xxx",
                    "name": "Dude Lightbulb",
                    "resource_type": "user",
                },
            },
        ),
        RuleTest(
            name="Azure - Rogue State Sign-In",
            expected_result=True,
            mocks=[RuleMock(object_name="is_rogue_state", return_value=True)],
            log={
                "calleripaddress": "12.12.12.12",
                "category": "ServicePrincipalSignInLogs",
                "correlationid": "bf12205b-eea0-43dd-ad6d-b9030dc62a7a",
                "durationms": 0,
                "level": 4,
                "location": "US",
                "operationname": "Sign-in activity",
                "operationversion": 1,
                "p_enrichment": {
                    "ipinfo_location": {
                        "properties.ipAddress": {
                            "city": "Moscow",
                            "country": "RU",
                            "lat": "55.7520",
                            "lng": "37.6150",
                            "p_match": "1.1.1.1",
                            "postal_code": "119021",
                            "region": "Moscow",
                            "region_code": "RU",
                            "timezone": "Europe/Moscow",
                        },
                    },
                },
                "p_log_type": "Azure.Audit",
                "properties": {
                    "appId": "3b245ca3-dcce-4a54-a070-49ad8de02963",
                    "authenticationProcessingDetails": [
                        {
                            "key": "Azure AD App Authentication Library",
                            "value": "Family: Unknown Library: Unknown 1.0.0 Platform: Unknown",
                        },
                    ],
                    "authenticationProtocol": "none",
                    "clientCredentialType": "none",
                    "conditionalAccessStatus": "notApplied",
                    "correlationId": "52d1e530-786c-443c-8dc8-aa7b1317608e",
                    "createdDateTime": "2023-07-27 13:59:53.691680300",
                    "crossTenantAccessType": "none",
                    "flaggedForReview": False,
                    "id": "55270060-d8fe-435e-9bf2-219a1d456b60",
                    "incomingTokenType": "none",
                    "ipAddress": "12.12.12.12",
                    "isInteractive": False,
                    "isTenantRestricted": False,
                    "location": {
                        "city": "Springfield",
                        "countryOrRegion": "US",
                        "geoCoordinates": {"latitude": 42.73333333333333, "longitude": -110.88888888888889},
                        "state": "Oregon",
                    },
                    "managedIdentityType": "none",
                    "processingTimeInMilliseconds": 0,
                    "resourceDisplayName": "Office 365 Management APIs",
                    "resourceId": "9cc31481-8822-49ff-b638-552ecc26c777",
                    "resourceServicePrincipalId": "2acf8174-5e07-4ec4-9de8-08d880129ba5",
                    "riskDetail": "none",
                    "riskLevelAggregated": "low",
                    "riskLevelDuringSignIn": "low",
                    "riskState": "none",
                    "servicePrincipalCredentialKeyId": "2afc776a-4e79-4588-b2ad-f62c94d6bea8",
                    "servicePrincipalId": "4b6986ec-c49c-40c0-89ce-b2ac51213e39",
                    "servicePrincipalName": "very-normal-service-principal",
                    "status": {"errorCode": 0},
                    "tokenIssuerType": "AzureAD",
                    "uniqueTokenIdentifier": "CXXXXXXXXXXXXXXXXXXXXX",
                },
                "resourceid": "/tenants/60641ed1-32f7-4a2e-a912-d724c497e1e9/providers/Microsoft.aadiam",
                "resultsignature": "None",
                "resulttype": 0,
                "tenantid": "85e54ec3-85ee-4b03-9e3b-863075eb9b62",
                "time": "2023-07-27 14:00:41.848",
            },
        ),
        RuleTest(
            name="Box - Rogue State Sign-In",
            expected_result=True,
            mocks=[RuleMock(object_name="is_rogue_state", return_value=True)],
            log={
                "type": "event",
                "additional_details": '{"key": "value"}',
                "created_by": {"id": "12345678", "type": "user", "login": "cat@example", "name": "Bob Cat"},
                "event_type": "LOGIN",
                "ip_address": "1.1.1.1",
                "p_enrichment": {
                    "ipinfo_location": {
                        "ip_address": {
                            "city": "Moscow",
                            "country": "RU",
                            "lat": "55.7520",
                            "lng": "37.6150",
                            "p_match": "1.1.1.1",
                            "postal_code": "119021",
                            "region": "Moscow",
                            "region_code": "RU",
                            "timezone": "Europe/Moscow",
                        },
                    },
                },
                "p_log_type": "Box.Event",
                "source": {"id": "12345678", "type": "user", "login": "user@example"},
            },
        ),
        RuleTest(
            name="Cloudtrail - Rogue State Sign-In",
            expected_result=True,
            mocks=[RuleMock(object_name="is_rogue_state", return_value=True)],
            log={
                "additionalEventData": {"MFAUsed": "No", "MobileVersion": "No"},
                "awsRegion": "us-east-1",
                "eventCategory": "Management",
                "eventName": "ConsoleLogin",
                "eventSource": "signin.amazonaws.com",
                "eventTime": "2023-05-26 20:14:51",
                "eventType": "AwsConsoleSignIn",
                "eventVersion": "1.08",
                "managementEvent": True,
                "p_event_time": "2023-05-26 20:14:51",
                "p_enrichment": {
                    "ipinfo_location": {
                        "load_ip_address": {
                            "city": "Moscow",
                            "country": "RU",
                            "lat": "55.7520",
                            "lng": "37.6150",
                            "p_match": "1.1.1.1",
                            "postal_code": "119021",
                            "region": "Moscow",
                            "region_code": "RU",
                            "timezone": "Europe/Moscow",
                        },
                    },
                },
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "2023-05-26 20:19:14.002",
                "p_source_label": "LogSource Name",
                "readOnly": False,
                "recipientAccountId": "123456789012",
                "responseElements": {"ConsoleLogin": "Success"},
                "sourceIPAddress": "12.12.12.12",
                "tlsDetails": {
                    "cipherSuite": "TLS_AES_128_GCM_SHA256",
                    "clientProvidedHostHeader": "signin.aws.amazon.com",
                    "tlsVersion": "TLSv1.3",
                },
                "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                "userIdentity": {
                    "type": "IAMUser",
                    "principalId": "1111",
                    "arn": "arn:aws:iam::123456789012:user/tester",
                    "accountId": "123456789012",
                    "userName": "tester",
                },
            },
        ),
        RuleTest(
            name="Notion - Rogue State Sign-In",
            expected_result=True,
            mocks=[RuleMock(object_name="is_rogue_state", return_value=True)],
            log={
                "event": {
                    "actor": {
                        "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                        "object": "user",
                        "person": {"email": "aragorn.elessar@lotr.com"},
                        "type": "person",
                    },
                    "details": {"authType": "saml"},
                    "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                    "ip_address": "192.168.100.100",
                    "timestamp": "2023-10-03T19:02:28.044000Z",
                    "type": "user.login",
                    "workspace_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                },
                "p_enrichment": {
                    "ipinfo_location": {
                        "event.ip_address": {
                            "city": "Moscow",
                            "country": "RU",
                            "lat": "55.7520",
                            "lng": "37.6150",
                            "p_match": "1.1.1.1",
                            "postal_code": "119021",
                            "region": "Moscow",
                            "region_code": "RU",
                            "timezone": "Europe/Moscow",
                        },
                    },
                },
                "p_event_time": "2023-10-03T19:02:28.044000Z",
                "p_log_type": "Notion.AuditLogs",
                "p_source_label": "Notion-Panther-Labs",
            },
        ),
        RuleTest(
            name="Okta - Rogue State Sign-In",
            expected_result=True,
            mocks=[RuleMock(object_name="is_rogue_state", return_value=True)],
            log={
                "actor": {
                    "alternateId": "dude.lightbulb@example.co",
                    "displayName": "Dude Lightbulb",
                    "id": "EXAMPLE_USER_ID",
                    "type": "User",
                },
                "authenticationContext": {"authenticationStep": 0, "externalSessionId": "EXAMPLE_SESSION_ID"},
                "client": {
                    "device": "Computer",
                    "geographicalContext": {
                        "city": "Moscow",
                        "country": "Russia",
                        "geolocation": {"lat": 55.752, "lon": 37.615},
                        "postalCode": "119021",
                        "state": "Moscow",
                    },
                    "ipAddress": "1.1.1.1",
                    "userAgent": {
                        "browser": "CHROME",
                        "os": "Mac OS X",
                        "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36",
                    },
                    "zone": "null",
                },
                "debugContext": {
                    "debugData": {
                        "authnRequestId": "xxx",
                        "deviceFingerprint": "xxx",
                        "dtHash": "xxx",
                        "logOnlySecurityData": '{"risk":{"level":"LOW"},"behaviors":{"New Geo-Location":"NEGATIVE","New Device":"NEGATIVE","New IP":"NEGATIVE","New State":"NEGATIVE","New Country":"NEGATIVE","Velocity":"NEGATIVE","New City":"NEGATIVE"}}',
                        "oktaUserAgentExtended": "okta-auth-js/7.7.0 okta-signin-widget-7.21.0",
                        "origin": "https://example.okta.com",
                        "requestId": "xxx",
                        "requestUri": "/idp/idx/identify",
                        "threatSuspected": "false",
                        "url": "/idp/idx/identify?",
                    },
                },
                "displayMessage": "User login to Okta",
                "eventType": "user.session.start",
                "legacyEventType": "core.user_auth.login_success",
                "outcome": {"result": "SUCCESS"},
                "p_enrichment": {
                    "ipinfo_location": {
                        "client.ipAddress": {
                            "city": "Moscow",
                            "country": "RU",
                            "lat": "55.7520",
                            "lng": "37.6150",
                            "p_match": "1.1.1.1",
                            "postal_code": "119021",
                            "region": "Moscow",
                            "region_code": "RU",
                            "timezone": "Europe/Moscow",
                        },
                    },
                },
                "p_event_time": "2024-08-15 15:05:09.154000000",
                "p_log_type": "Okta.SystemLog",
                "p_parse_time": "2024-08-15 15:08:22.176160519",
                "p_row_id": "526383f6c367e7f3db88959621afee19",
                "p_source_id": "d0907120-58a3-4e40-acfa-e631693f9066",
                "p_source_label": "My Log Source",
                "published": "2024-08-15 15:05:09.154000000",
                "request": {
                    "ipChain": [
                        {
                            "geographicalContext": {
                                "city": "Moscow",
                                "country": "Russia",
                                "geolocation": {"lat": 55.752, "lon": 37.615},
                                "postalCode": "119021",
                                "state": "Moscow",
                            },
                            "ip": "1.1.1.1",
                            "version": "V4",
                        },
                    ],
                },
                "securityContext": {
                    "asNumber": 7122,
                    "asOrg": "SAMPLE_ISP",
                    "domain": "isp.net",
                    "isProxy": False,
                    "isp": "SAMPLE_ISP",
                },
                "severity": "INFO",
                "target": [
                    {"alternateId": "My Okta App", "displayName": "My Okta App", "id": "xxx", "type": "AppInstance"},
                ],
                "transaction": {"detail": {}, "id": "32caf8cb5819a0928702b4b835e163a0", "type": "WEB"},
                "uuid": "c35900e7-5b17-11ef-ad6d-cf78a9534d8d",
                "version": "0",
            },
        ),
        RuleTest(
            name="OneLogin - Rogue State Sign-In",
            expected_result=True,
            mocks=[RuleMock(object_name="is_rogue_state", return_value=True)],
            log={
                "event_type_id": "5",
                "actor_user_id": 123456,
                "actor_user_name": "Bob Cat",
                "ipaddr": "1.1.1.1",
                "p_enrichment": {
                    "ipinfo_location": {
                        "ipaddr": {
                            "city": "Moscow",
                            "country": "RU",
                            "lat": "55.7520",
                            "lng": "37.6150",
                            "p_match": "1.1.1.1",
                            "postal_code": "119021",
                            "region": "Moscow",
                            "region_code": "RU",
                            "timezone": "Europe/Moscow",
                        },
                    },
                },
                "p_log_type": "OneLogin.Events",
                "user_id": 123456,
                "user_name": "Bob Cat",
            },
        ),
        RuleTest(
            name="OnePassword - Rogue State Sign-In",
            expected_result=True,
            mocks=[RuleMock(object_name="is_rogue_state", return_value=True)],
            log={
                "category": "success",
                "client": {
                    "app_name": "1Password Browser Extension",
                    "app_version": "22600103",
                    "ip_address": "1.1.1.1",
                    "os_name": "MacOSX",
                    "os_version": "10.15",
                    "platform_name": "Firefox extension",
                    "platform_version": "128.0",
                },
                "country": "RU",
                "location": {
                    "city": "Moscow",
                    "country": "RU",
                    "latitude": 55.752,
                    "longitude": 37.615,
                    "region": "Moscow",
                },
                "p_enrichment": {
                    "ipinfo_location": {
                        "client.ip_address": {
                            "city": "Moscow",
                            "country": "RU",
                            "lat": "55.7520",
                            "lng": "37.6150",
                            "p_match": "1.1.1.1",
                            "postal_code": "119021",
                            "region": "Moscow",
                            "region_code": "RU",
                            "timezone": "Europe/Moscow",
                        },
                    },
                },
                "p_event_time": "2024-08-16 19:18:43.784550714",
                "p_log_type": "OnePassword.SignInAttempt",
                "p_parse_time": "2024-08-16 19:28:04.114167542",
                "p_row_id": "ded6c31fcf859c8fd1f7fc9821e3b703",
                "p_schema_version": 0,
                "p_source_id": "9f722ac4-3715-4db8-a0b4-a62f34599f90",
                "p_source_label": "1Password",
                "session_uuid": "EXAMPLE_SESSION_ID",
                "target_user": {
                    "email": "dude.lightbulb@example.co",
                    "name": "Dude Lightbulb",
                    "uuid": "EXAMPLE_PROVIDER_ID",
                },
                "timestamp": "2024-08-16 19:18:43.784550714",
                "type": "credentials_ok",
                "uuid": "EXAMPLE_UUID",
            },
        ),
        RuleTest(
            name="Zoom - Rogue State Sign-In",
            expected_result=True,
            mocks=[RuleMock(object_name="is_rogue_state", return_value=True)],
            log={
                "client_type": "mac",
                "email": "dude.lightbulb@example.co",
                "ip_address": "1.1.1.1",
                "p_enrichment": {
                    "ipinfo_location": {
                        "ip_address": {
                            "city": "Moscow",
                            "country": "RU",
                            "lat": "55.7520",
                            "lng": "37.6150",
                            "p_match": "1.1.1.1",
                            "postal_code": "119021",
                            "region": "Moscow",
                            "region_code": "RU",
                            "timezone": "Europe/Moscow",
                        },
                    },
                },
                "p_event_time": "2023-08-14 16:17:05",
                "p_log_type": "Zoom.Activity",
                "p_parse_time": "2023-08-14 16:22:14.232",
                "p_row_id": "d2cc7476479cc4b1e3a0bbfb1901",
                "p_schema_version": 0,
                "p_source_id": "afedae15-43e8-45d1-85e6-8f525e176d3e",
                "p_source_label": "Zoom",
                "time": "2023-08-14 16:17:05",
                "type": "Sign in",
                "version": "5.15.7.21404",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class SnowflakeStreamPublicRoleGrant(Rule):
    id = "Snowflake.Stream.PublicRoleGrant-prototype"
    display_name = "Snowflake Grant to Public Role"
    log_types = [LogType.SNOWFLAKE_GRANTS_TO_ROLES]
    default_severity = Severity.MEDIUM
    reports = {"MITRE ATT&CK": ["TA0004:T1078.001"]}
    default_description = "Detect additional grants to the public role."
    default_runbook = (
        "Determine if this is a necessary grant for the public role, which should be kept to the fewest possible."
    )
    tags = [
        "Snowflake",
        "[MITRE] Privilege Escalation",
        "[MITRE] Valid Accounts",
        "[MITRE] Valid Accounts: Default Accounts",
    ]

    def rule(self, event):
        return event.get("GRANTEE_NAME").lower() == "public"

    def title(self, event):
        return f"{event.get('p_source_label', '<UNKNOWN LOG SOURCE>')}: {event.get('GRANTED_BY', '<UNKNWON ACTOR>')} made changes to the PUBLIC role"

    tests = [
        RuleTest(
            name="SELECT Granted to Public",
            expected_result=True,
            log={
                "p_source_label": "DailyPlanet-Snowflake",
                "CREATED_ON": "2024-10-10 12:56:35.822 -0700",
                "MODIFIED_ON": "2024-10-10 12:56:35.822 -0700",
                "PRIVILEGE": "SELECT",
                "GRANTED_ON": "TABLE",
                "NAME": "MYTABLE",
                "TABLE_CATALOG": "TEST_DB",
                "TABLE_SCHEMA": "PUBLIC",
                "GRANTED_TO": "ROLE",
                "GRANTEE_NAME": "PUBLIC",
                "GRANT_OPTION": False,
                "GRANTED_BY": "ACCOUNTADMIN",
                "DELETED_ON": "",
                "GRANTED_BY_ROLE_TYPE": "ROLE",
                "OBJECT_INSTANCE": "",
            },
        ),
        RuleTest(
            name="Privilege Granted to Non-PUBLIC Role",
            expected_result=False,
            log={
                "p_source_label": "DailyPlanet-Snowflake",
                "CREATED_ON": "2024-10-10 12:56:35.822 -0700",
                "MODIFIED_ON": "2024-10-10 12:56:35.822 -0700",
                "PRIVILEGE": "SELECT",
                "GRANTED_ON": "TABLE",
                "NAME": "MYTABLE",
                "TABLE_CATALOG": "TEST_DB",
                "TABLE_SCHEMA": "PUBLIC",
                "GRANTED_TO": "ROLE",
                "GRANTEE_NAME": "APP_READONLY",
                "GRANT_OPTION": False,
                "GRANTED_BY": "ACCOUNTADMIN",
                "DELETED_ON": "",
                "GRANTED_BY_ROLE_TYPE": "ROLE",
                "OBJECT_INSTANCE": "",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.thinkstcanary import additional_details


@panther_managed
class ThinkstCanaryTokenIncident(Rule):
    display_name = "Thinkst Canarytoken Incident"
    id = "Thinkst.CanaryTokenIncident-prototype"
    default_description = "A Canarytoken incident has been detected."
    default_severity = Severity.HIGH
    log_types = [LogType.THINKSTCANARY_ALERT]

    def rule(self, event):
        return event.get("AlertType") == "CanarytokenIncident"

    def title(self, event):
        return event.get("Intro", "Canary Token Incident")

    def alert_context(self, event):
        return additional_details(event)

    tests = [
        RuleTest(
            name="Canarytoken Incident",
            expected_result=True,
            log={
                "AdditionalDetails": [
                    ["Background Context", "You have had 4 incidents from 123.123.123.123 previously."],
                    ["Dst Port", 80],
                    ["Event Name", "GetCallerIdentity"],
                    ["User-Agent", "TruffleHog"],
                ],
                "AlertType": "CanarytokenIncident",
                "Description": "AWS API Key Canarytoken triggered",
                "IncidentHash": "79cb967bde35e3b2d3b346844c16c4bf",
                "IncidentKey": "incident:canarytoken:94e08d45e5f2c8c13e7b99ae:123.123.123.123:1718797361",
                "Intro": "An AWS API Key Canarytoken was triggered by '123.123.123.123'.",
                "MatchedAnnotations": {
                    "trufflehog_scan": [
                        "This looks like a TruffleHog scan.",
                        "https://help.canary.tools/hc/en-gb/articles/18185364902813-Alert-Annotation-TruffleHog-Scan",
                    ],
                },
                "Reminder": "aws api key inside keepass",
                "SourceIP": "123.123.123.123",
                "Timestamp": "2024-06-19 11:42:41 (UTC)",
                "Token": "jf15ldk2jeaooi8dhlc6rgt9g",
                "Triggered": "2",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.mongodb import mongodb_alert_context


@panther_managed
class MongoDBAlertingDisabledOrDeleted(Rule):
    default_description = "MongoDB provides security alerting policies for notifying admins when certain conditions are met. This rule detects when these policies are disabled or deleted."
    display_name = "MongoDB security alerts disabled or deleted"
    log_types = [LogType.MONGODB_ORGANIZATION_EVENT]
    id = "MongoDB.Alerting.Disabled.Or.Deleted-prototype"
    default_severity = Severity.HIGH
    reports = {"MITRE ATT&CK": ["TA0005:T1562.001"]}
    default_reference = "https://www.mongodb.com/docs/atlas/configure-alerts/"
    default_runbook = "Re-enable security alerts"

    def rule(self, event):
        return event.get("eventTypeName", "") in ["ALERT_CONFIG_DISABLED_AUDIT", "ALERT_CONFIG_DELETED_AUDIT"]

    def title(self, event):
        user = event.get("username", "<USER_NOT_FOUND>")
        alert_id = event.get("alertConfigId", "<ALERT_NOT_FOUND>")
        return f"MongoDB: [{user}] has disabled or deleted security alert [{alert_id}]"

    def alert_context(self, event):
        context = mongodb_alert_context(event)
        context["alertConfigId"] = event.get("alertConfigId", "<ALERT_NOT_FOUND>")
        return context

    tests = [
        RuleTest(
            name="Alert added",
            expected_result=False,
            log={
                "alertConfigId": "alert_id",
                "created": "2024-04-01 11:57:54.000000000",
                "currentValue": {},
                "eventTypeName": "ALERT_CONFIG_ADDED_AUDIT",
                "id": "alert_id",
                "isGlobalAdmin": False,
                "links": [],
                "orgId": "some_org_id",
                "remoteAddress": "1.2.3.4",
                "userId": "user_id",
                "username": "some_user@company.com",
            },
        ),
        RuleTest(
            name="Alert deleted",
            expected_result=True,
            log={
                "alertConfigId": "alert_id",
                "created": "2024-04-01 11:58:52.000000000",
                "currentValue": {},
                "eventTypeName": "ALERT_CONFIG_DELETED_AUDIT",
                "id": "alert_id",
                "isGlobalAdmin": False,
                "links": [],
                "orgId": "some_org_id",
                "remoteAddress": "1.2.3.4",
                "userId": "user_id",
                "username": "some_user@company.com",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.gcp import gcp_alert_context


@panther_managed
class GCPProjectExternalUserOwnershipInvite(Rule):
    display_name = "GCP External User Ownership Invite"
    id = "GCP.Project.ExternalUserOwnershipInvite-prototype"
    default_severity = Severity.HIGH
    log_types = [LogType.GCP_AUDIT_LOG]
    default_description = "This rule detects when an external user is invited as an owner of a GCP project using the InsertProjectOwnershipInvite event.\n"
    default_runbook = "Investigate the invitation to ensure it was authorized. Unauthorized invitations can lead to security risks. If the invitation was unauthorized, revoke the user's access to the project.\n"
    default_reference = "https://cloud.google.com/resource-manager/docs/project-ownership"

    def rule(self, event):
        if event.deep_get("protoPayload", "response", "error"):
            return False
        method = event.deep_get("protoPayload", "methodName", default="METHOD_NOT_FOUND")
        if method != "InsertProjectOwnershipInvite":
            return False
        authenticated = event.deep_get("protoPayload", "authenticationInfo", "principalEmail", default="")
        expected_domain = authenticated.split("@")[-1]
        if event.deep_get("protoPayload", "request", "member", default="MEMBER_NOT_FOUND").endswith(
            f"@{expected_domain}",
        ):
            return False
        return True

    def title(self, event):
        member = event.deep_get("protoPayload", "request", "member", default="<MEMBER_NOT_FOUND>")
        project = event.deep_get("protoPayload", "resourceName", default="<PROJECT_NOT_FOUND>")
        return f"[GCP]: External user [{member}] was invited as owner to project [{project}]"

    def alert_context(self, event):
        return gcp_alert_context(event)

    tests = [
        RuleTest(
            name="External User Ownership Invite",
            expected_result=True,
            log={
                "insertId": "1abcd23efg456",
                "logName": "projects/test-project/logs/cloudaudit.googleapis.com%2Factivity",
                "protoPayload": {
                    "@type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "methodName": "InsertProjectOwnershipInvite",
                    "resourceName": "projects/target-project",
                    "authenticationInfo": {"principalEmail": "user@runpanther.com"},
                    "request": {
                        "member": "user:attacker@gmail.com",
                        "projectId": "target-project",
                        "@type": "type.googleapis.com/google.internal.cloud.resourcemanager.InsertProjectOwnershipInviteRequest",
                    },
                    "response": {
                        "@type": "type.googleapis.com/google.internal.cloud.resourcemanager.InsertProjectOwnershipInviteResponse",
                    },
                    "serviceName": "cloudresourcemanager.googleapis.com",
                },
                "resource": {"labels": {"project_id": "target-project"}, "type": "gce_project"},
                "severity": "NOTICE",
                "timestamp": "2023-10-01T12:34:56.789Z",
            },
        ),
        RuleTest(
            name="Internal User Ownership Invite",
            expected_result=False,
            log={
                "insertId": "2hijk34lmn789",
                "logName": "projects/test-project/logs/cloudaudit.googleapis.com%2Factivity",
                "protoPayload": {
                    "@type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "methodName": "InsertProjectOwnershipInvite",
                    "resourceName": "projects/target-project",
                    "authenticationInfo": {"principalEmail": "user@runpanther.com"},
                    "request": {
                        "member": "user:internal-user@runpanther.com",
                        "projectId": "target-project",
                        "@type": "type.googleapis.com/google.internal.cloud.resourcemanager.InsertProjectOwnershipInviteRequest",
                    },
                    "response": {
                        "@type": "type.googleapis.com/google.internal.cloud.resourcemanager.InsertProjectOwnershipInviteResponse",
                    },
                    "serviceName": "cloudresourcemanager.googleapis.com",
                },
                "resource": {"labels": {"project_id": "target-project"}, "type": "gce_project"},
                "severity": "NOTICE",
                "timestamp": "2023-10-01T12:45:56.789Z",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers import event_type


@panther_managed
class StandardMFADisabled(Rule):
    id = "Standard.MFADisabled-prototype"
    display_name = "MFA Disabled"
    log_types = [LogType.ATLASSIAN_AUDIT, LogType.GITHUB_AUDIT, LogType.ZENDESK_AUDIT, LogType.OKTA_SYSTEM_LOG]
    tags = ["DataModel", "Defense Evasion:Modify Authentication Process"]
    reports = {"MITRE ATT&CK": ["TA0005:T1556"]}
    default_reference = "https://en.wikipedia.org/wiki/Multi-factor_authentication"
    default_severity = Severity.HIGH
    default_description = "Detects when Multi-Factor Authentication (MFA) is disabled"
    summary_attributes = ["p_any_ip_addresses"]

    def rule(self, event):
        return event.udm("event_type") == event_type.MFA_DISABLED

    def title(self, event):
        # use unified data model field in title
        return f"{event.get('p_log_type')}: User [{event.udm('actor_user')}] disabled MFA"

    tests = [
        RuleTest(
            name="GitHub - Org MFA Disabled",
            expected_result=True,
            log={
                "actor": "cat",
                "action": "org.disable_two_factor_requirement",
                "created_at": 1621305118553,
                "org": "my-org",
                "p_log_type": "GitHub.Audit",
                "repository": "my-org/my-repo",
            },
        ),
        RuleTest(
            name="GitHub - Org MFA Enabled",
            expected_result=False,
            log={
                "actor": "cat",
                "action": "org.enable_two_factor_requirement",
                "created_at": 1621305118553,
                "org": "my-org",
                "p_log_type": "GitHub.Audit",
                "repository": "my-org/my-repo",
            },
        ),
        RuleTest(
            name="Zendesk - Two-factor disabled",
            expected_result=True,
            log={
                "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
                "id": 123456789123,
                "action_label": "Updated",
                "actor_id": 123,
                "source_id": 123,
                "source_type": "account_setting",
                "source_label": "Two-Factor authentication for all admins and agents",
                "action": "create",
                "change_description": "Disabled",
                "ip_address": "127.0.0.1",
                "created_at": "2021-05-28T18:39:50Z",
                "p_log_type": "Zendesk.Audit",
            },
        ),
        RuleTest(
            name="Zendesk - Alternate Two-factor disabled",
            expected_result=True,
            log={
                "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
                "id": 123456789123,
                "action_label": "Updated",
                "actor_id": 123,
                "source_id": 123,
                "source_type": "account_setting",
                "source_label": "Require Two Factor",
                "action": "update",
                "change_description": "Disabled",
                "ip_address": "127.0.0.1",
                "created_at": "2021-05-28T18:39:50Z",
                "p_log_type": "Zendesk.Audit",
            },
        ),
        RuleTest(
            name="Zendesk - Two-factor Enabled",
            expected_result=False,
            log={
                "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
                "id": 123456789123,
                "action_label": "Updated",
                "actor_id": 123,
                "source_id": 123,
                "source_type": "account_setting",
                "source_label": "Require Two Factor",
                "action": "create",
                "change_description": "Enabled",
                "ip_address": "127.0.0.1",
                "created_at": "2021-05-28T18:39:50Z",
                "p_log_type": "Zendesk.Audit",
            },
        ),
        RuleTest(
            name="Okta - User resets own MFA ( INFO severity detection for this in Okta rules )",
            expected_result=False,
            log={
                "eventtype": "user.mfa.factor.deactivate",
                "version": "0",
                "severity": "INFO",
                "displaymessage": "Reset factor for user",
                "actor": {
                    "alternateId": "homer@springfield.gov",
                    "displayName": "Homer Simpson",
                    "id": "11111111111",
                    "type": "User",
                },
                "client": {
                    "device": "Computer",
                    "ipAddress": "1.1.1.1",
                    "userAgent": {
                        "browser": "CHROME",
                        "os": "Mac OS X",
                        "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/96.0.4664.55 Safari/537.36",
                    },
                    "zone": "null",
                },
                "outcome": {"reason": "User reset FIDO_WEBAUTHN factor", "result": "SUCCESS"},
                "target": [
                    {
                        "alternateId": "homer@springfield.gov",
                        "displayName": "Homer Simpson",
                        "id": "1111111",
                        "type": "User",
                    },
                ],
                "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "1111111"},
                "p_log_type": "Okta.SystemLog",
            },
        ),
        RuleTest(
            name="Okta - User disables MFA",
            expected_result=True,
            log={
                "eventtype": "user.mfa.factor.deactivate",
                "version": "0",
                "severity": "INFO",
                "displaymessage": "Reset factor for user",
                "actor": {
                    "alternateId": "homer@springfield.gov",
                    "displayName": "Homer Simpson",
                    "id": "11111111111",
                    "type": "User",
                },
                "client": {
                    "device": "Computer",
                    "ipAddress": "1.1.1.1",
                    "userAgent": {
                        "browser": "CHROME",
                        "os": "Mac OS X",
                        "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/96.0.4664.55 Safari/537.36",
                    },
                    "zone": "null",
                },
                "outcome": {"result": "SUCCESS"},
                "target": [
                    {
                        "alternateId": "homer@springfield.gov",
                        "displayName": "Homer Simpson",
                        "id": "1111111",
                        "type": "User",
                    },
                ],
                "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "1111111"},
                "p_log_type": "Okta.SystemLog",
            },
        ),
        RuleTest(
            name="Okta - MFA Enabled",
            expected_result=False,
            log={
                "eventtype": "user.mfa.factor.update",
                "version": "0",
                "severity": "INFO",
                "displaymessage": "Update factor for user",
                "actor": {
                    "alternateId": "homer@springfield.gov",
                    "displayName": "Homer Simpson",
                    "id": "11111111111",
                    "type": "User",
                },
                "client": {
                    "device": "Computer",
                    "ipAddress": "1.1.1.1",
                    "userAgent": {
                        "browser": "CHROME",
                        "os": "Mac OS X",
                        "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/96.0.4664.55 Safari/537.36",
                    },
                    "zone": "null",
                },
                "outcome": {"reason": "User reset FIDO_WEBAUTHN factor", "result": "SUCCESS"},
                "target": [
                    {
                        "alternateId": "homer@springfield.gov",
                        "displayName": "Homer Simpson",
                        "id": "1111111",
                        "type": "User",
                    },
                ],
                "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "1111111"},
                "p_log_type": "Okta.SystemLog",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.cloudflare import cloudflare_http_alert_context


@panther_managed
class CloudflareHttpRequestBotHighVolume(Rule):
    id = "Cloudflare.HttpRequest.BotHighVolume-prototype"
    display_name = "Cloudflare Bot High Volume"
    enabled = False
    log_types = [LogType.CLOUDFLARE_HTTP_REQUEST]
    tags = ["Cloudflare"]
    default_severity = Severity.LOW
    default_description = "Monitors for bots making HTTP Requests at a rate higher than 2req/sec"
    default_runbook = "Inspect and monitor internet-facing services for potential outages"
    default_reference = "https://developers.cloudflare.com/waf/rate-limiting-rules/request-rate/"
    threshold = 7560
    summary_attributes = [
        "ClientIP",
        "ClientRequestUserAgent",
        "EdgeResponseContentType",
        "ClientCountry",
        "ClientRequestURI",
    ]

    def rule(self, event):
        # Bot scores are [0, 99] where scores >0 && <30 indicating likely automated
        # https://developers.cloudflare.com/bots/concepts/bot-score/
        return all([event.get("BotScore", 100) <= 30, event.get("BotScore", 100) >= 1])

    def title(self, event):
        return f"Cloudflare: High Volume of Bot Requests - from [{event.get('ClientIP', '<NO_CLIENTIP>')}] to [{event.get('ClientRequestHost', '<NO_REQ_HOST>')}]"

    def dedup(self, event):
        return f"{event.get('ClientIP', '<NO_CLIENTIP>')}:{event.get('ClientRequestHost', '<NO_REQ_HOST>')}"

    def alert_context(self, event):
        return cloudflare_http_alert_context(event)

    tests = [
        RuleTest(
            name="Likely Human",
            expected_result=False,
            log={
                "BotScore": 99,
                "CacheCacheStatus": "miss",
                "CacheResponseBytes": 76931,
                "CacheResponseStatus": 404,
                "CacheTieredFill": False,
                "ClientASN": 63949,
                "ClientCountry": "gb",
                "ClientDeviceType": "desktop",
                "ClientIP": "142.93.204.250",
                "ClientIPClass": "noRecord",
                "ClientRequestBytes": 2407,
                "ClientRequestHost": "example.com",
                "ClientRequestMethod": "GET",
                "ClientRequestPath": "",
                "ClientRequestProtocol": "HTTP/1.1",
                "ClientRequestReferer": "https://example.com/",
                "ClientRequestURI": "",
                "ClientRequestUserAgent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/83.0.4103.97 Safari/537.36",
                "ClientSSLProtocol": "TLSv1.3",
                "ClientSrcPort": 28057,
                "ClientXRequestedWith": "",
                "EdgeColoCode": "LHR",
                "EdgeColoID": 373,
                "EdgeEndTimestamp": "2022-05-07 18:53:13",
                "EdgePathingOp": "wl",
                "EdgePathingSrc": "macro",
                "EdgePathingStatus": "nr",
                "EdgeRateLimitAction": "",
                "EdgeRateLimitID": "0",
                "EdgeRequestHost": "example.com",
                "EdgeResponseBytes": 17826,
                "EdgeResponseCompressionRatio": 4.55,
                "EdgeResponseContentType": "text/html",
                "EdgeResponseStatus": 404,
                "EdgeServerIP": "",
                "EdgeStartTimestamp": "2022-05-07 18:53:12",
                "OriginIP": "",
                "OriginResponseBytes": 0,
                "OriginResponseStatus": 0,
                "OriginResponseTime": 0,
                "OriginSSLProtocol": "unknown",
                "ParentRayID": "00",
                "RayID": "707c283ab88274cd",
                "SecurityLevel": "med",
                "WAFAction": "unknown",
                "WAFFlags": "0",
                "WAFMatchedVar": "",
                "WAFProfile": "unknown",
                "WAFRuleID": "",
                "WAFRuleMessage": "",
                "WorkerCPUTime": 0,
                "WorkerStatus": "unknown",
                "WorkerSubrequest": False,
                "WorkerSubrequestCount": 0,
                "ZoneID": 526503649,
                "p_any_domain_names": ["https://example.com/", "example.com"],
                "p_any_ip_addresses": ["142.93.204.250"],
                "p_any_trace_ids": ["00", "707c283ab88274cd"],
                "p_event_time": "2022-05-07 18:53:12",
                "p_log_type": "Cloudflare.HttpRequest",
                "p_parse_time": "2022-05-07 18:54:31.922",
                "p_row_id": "a6e3965df054cfcdbdccf3ec10a134",
                "p_source_id": "2b9fc5ae-9cab-4715-8683-9bfbdb15a313",
                "p_source_label": "Cloudflare",
            },
        ),
        RuleTest(
            name="Likely Automated",
            expected_result=True,
            log={
                "BotScore": 29,
                "CacheCacheStatus": "miss",
                "CacheResponseBytes": 76931,
                "CacheResponseStatus": 404,
                "CacheTieredFill": False,
                "ClientASN": 63949,
                "ClientCountry": "gb",
                "ClientDeviceType": "desktop",
                "ClientIP": "142.93.204.250",
                "ClientIPClass": "noRecord",
                "ClientRequestBytes": 2407,
                "ClientRequestHost": "example.com",
                "ClientRequestMethod": "GET",
                "ClientRequestPath": "",
                "ClientRequestProtocol": "HTTP/1.1",
                "ClientRequestReferer": "https://example.com/",
                "ClientRequestURI": "",
                "ClientRequestUserAgent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/83.0.4103.97 Safari/537.36",
                "ClientSSLProtocol": "TLSv1.3",
                "ClientSrcPort": 28057,
                "ClientXRequestedWith": "",
                "EdgeColoCode": "LHR",
                "EdgeColoID": 373,
                "EdgeEndTimestamp": "2022-05-07 18:53:13",
                "EdgePathingOp": "wl",
                "EdgePathingSrc": "macro",
                "EdgePathingStatus": "nr",
                "EdgeRateLimitAction": "",
                "EdgeRateLimitID": "0",
                "EdgeRequestHost": "example.com",
                "EdgeResponseBytes": 17826,
                "EdgeResponseCompressionRatio": 4.55,
                "EdgeResponseContentType": "text/html",
                "EdgeResponseStatus": 404,
                "EdgeServerIP": "",
                "EdgeStartTimestamp": "2022-05-07 18:53:12",
                "OriginIP": "",
                "OriginResponseBytes": 0,
                "OriginResponseStatus": 0,
                "OriginResponseTime": 0,
                "OriginSSLProtocol": "unknown",
                "ParentRayID": "00",
                "RayID": "707c283ab88274cd",
                "SecurityLevel": "med",
                "WAFAction": "unknown",
                "WAFFlags": "0",
                "WAFMatchedVar": "",
                "WAFProfile": "unknown",
                "WAFRuleID": "",
                "WAFRuleMessage": "",
                "WorkerCPUTime": 0,
                "WorkerStatus": "unknown",
                "WorkerSubrequest": False,
                "WorkerSubrequestCount": 0,
                "ZoneID": 526503649,
                "p_any_domain_names": ["https://example.com/", "example.com"],
                "p_any_ip_addresses": ["142.93.204.250"],
                "p_any_trace_ids": ["00", "707c283ab88274cd"],
                "p_event_time": "2022-05-07 18:53:12",
                "p_log_type": "Cloudflare.HttpRequest",
                "p_parse_time": "2022-05-07 18:54:31.922",
                "p_row_id": "a6e3965df054cfcdbdccf3ec10a134",
                "p_source_id": "2b9fc5ae-9cab-4715-8683-9bfbdb15a313",
                "p_source_label": "Cloudflare",
            },
        ),
    ]



from pypanther.rules.zscaler.zia.zia_account_access_removal import ZIAAccountAccessRemoved as ZIAAccountAccessRemoved
from pypanther.rules.zscaler.zia.zia_additional_cloud_roles import ZIAAdditionalCloudRoles as ZIAAdditionalCloudRoles
from pypanther.rules.zscaler.zia.zia_backup_deleted import ZIABackupDeleted as ZIABackupDeleted
from pypanther.rules.zscaler.zia.zia_create_cloud_account import ZIACloudAccountCreated as ZIACloudAccountCreated
from pypanther.rules.zscaler.zia.zia_golden_restore_point_dropped import (
    ZIAGoldenRestorePointDropped as ZIAGoldenRestorePointDropped,
)
from pypanther.rules.zscaler.zia.zia_insecure_password_settings import (
    ZIAInsecurePasswordSettings as ZIAInsecurePasswordSettings,
)
from pypanther.rules.zscaler.zia.zia_log_streaming_disabled import ZIALogStreamingDisabled as ZIALogStreamingDisabled
from pypanther.rules.zscaler.zia.zia_logs_downloaded import ZIALogsDownloaded as ZIALogsDownloaded
from pypanther.rules.zscaler.zia.zia_password_expiration import ZIAPasswordExpiration as ZIAPasswordExpiration
from pypanther.rules.zscaler.zia.zia_trust_modification import ZIATrustModification as ZIATrustModification



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class GCPUserAddedtoIAPProtectedService(Rule):
    default_description = "A user has been granted access to a IAP protected service."
    display_name = "GCP User Added to IAP Protected Service"
    default_runbook = "Note: GCP logs all bindings everytime this event occurs, not just changes. Bindings should be reviewed to ensure no unintended users have been added. "
    default_reference = "https://cloud.google.com/iap/docs/managing-access"
    default_severity = Severity.LOW
    log_types = [LogType.GCP_AUDIT_LOG]
    id = "GCP.User.Added.to.IAP.Protected.Service-prototype"

    def rule(self, event):
        return (
            event.deep_get("protoPayload", "methodName", default="")
            == "google.cloud.iap.v1.IdentityAwareProxyAdminService.SetIamPolicy"
        )

    def title(self, event):
        actor = event.deep_get("protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        service = event.deep_get("protoPayload", "request", "resource", default="<RESOURCE_NOT_FOUND>")
        return f"GCP: [{actor}] modified user access to IAP Protected Service [{service}]"

    def alert_context(self, event):
        bindings = event.deep_get("protoPayload", "request", "policy", "bindings", default=[{}])
        return {"bindings": bindings}

    tests = [
        RuleTest(
            name="other",
            expected_result=False,
            log={
                "insertid": "abcdefghijklmn",
                "logname": "projects/gcp-project1/logs/cloudaudit.googleapis.com%2Factivity",
                "operation": {
                    "id": "1234567890123-gcp-project1:abcdefghijklmnopqrstuvwz",
                    "last": True,
                    "producer": "bigquery.googleapis.com",
                },
                "p_any_emails": ["user@company.io"],
                "p_any_ip_addresses": ["1.2.3.4"],
                "p_event_time": "2023-03-28 18:37:06.079",
                "p_log_type": "GCP.AuditLog",
                "p_parse_time": "2023-03-28 18:38:14.478",
                "p_row_id": "06bf03d9d5dfbadba981899e1787bf05",
                "p_schema_version": 0,
                "p_source_id": "964c7894-9a0d-4ddf-864f-0193438221d6",
                "p_source_label": "gcp-logsource",
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "user@company.io"},
                    "authorizationInfo": [
                        {
                            "granted": True,
                            "permission": "bigquery.tables.delete",
                            "resource": "projects/gcp-project1/datasets/test1/tables/newtable",
                        },
                    ],
                    "metadata": {
                        "@type": "type.googleapis.com/google.cloud.audit.BigQueryAuditMetadata",
                        "methodName": "google.cloud.bigquery.v2.JobService.InsertJob",
                        "requestMetadata": {
                            "callerIP": "1.2.3.4",
                            "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/111.0.0.0 Safari/537.36,gzip(gfe),gzip(gfe)",
                        },
                        "resourceName": "projects/gcp-project1/datasets/test1/tables/newtable",
                        "serviceName": "bigquery.googleapis.com",
                        "status": {},
                        "tableDeletion": {
                            "jobName": "projects/gcp-project1/jobs/bquxjob_5e4a0679_18729a639d7",
                            "reason": "QUERY",
                        },
                    },
                    "receivetimestamp": "2023-03-28 18:37:06.745",
                    "resource": {
                        "labels": {"dataset_id": "test1", "project_id": "gcp-project1"},
                        "type": "bigquery_dataset",
                    },
                    "severity": "NOTICE",
                    "timestamp": "2023-03-28 18:37:06.079",
                },
            },
        ),
        RuleTest(
            name="Other IAP Event",
            expected_result=True,
            log={
                "insertId": "46ee5sd38mw",
                "logName": "projects/gcp-project1/logs/cloudaudit.googleapis.com%2Factivity",
                "p_any_emails": ["staging@company.io"],
                "p_any_ip_addresses": ["1.2.3.4"],
                "p_event_time": "2023-04-25 19:20:57.024",
                "p_log_type": "GCP.AuditLog",
                "p_parse_time": "2023-04-25 19:22:14.743",
                "p_row_id": "b2e9b7f5dc85a69981fac2e417b6bb03",
                "p_schema_version": 0,
                "p_source_id": "5b77391b-afad-46c7-8ddc-b8e21d4726b3",
                "p_source_label": "gcplogsource2",
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "staging@company.io"},
                    "authorizationInfo": [
                        {
                            "granted": True,
                            "permission": "iap.webServices.setIamPolicy",
                            "resourceAttributes": {
                                "name": "projects/123456789012/iap_web/compute/services/7312383563505470445",
                                "service": "iap.googleapis.com",
                                "type": "iap.googleapis.com/WebService",
                            },
                        },
                    ],
                    "methodName": "google.cloud.iap.v1.IdentityAwareProxyAdminService.SetIamPolicy",
                    "request": {
                        "@type": "type.googleapis.com/google.iam.v1.SetIamPolicyRequest",
                        "policy": {"etag": "BwX6LgT4YMw="},
                        "resource": "projects/123456789012/iap_web/compute/services/7312383563505470445",
                    },
                    "requestMetadata": {
                        "callerIP": "1.2.3.4",
                        "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/112.0.0.0 Safari/537.36,gzip(gfe),gzip(gfe)",
                        "destinationAttributes": {},
                        "requestAttributes": {"auth": {}, "time": "2023-04-25T19:20:57.295723118Z"},
                    },
                    "resourceName": "projects/123456789012/iap_web/compute/services/7312383563505470445",
                    "response": {"@type": "type.googleapis.com/google.iam.v1.Policy", "etag": "BwX6LgXbpsw="},
                    "serviceName": "iap.googleapis.com",
                },
                "receiveTimestamp": "2023-04-25 19:20:58.16",
                "resource": {
                    "labels": {"backend_service_id": "", "location": "", "project_id": "gcp-project1"},
                    "type": "gce_backend_service",
                },
                "severity": "NOTICE",
                "timestamp": "2023-04-25 19:20:57.024",
            },
        ),
        RuleTest(
            name="Add User to IAP",
            expected_result=True,
            log={
                "insertId": "yyultvcrhy",
                "logName": "projects/gcp-projet1/logs/cloudaudit.googleapis.com%2Factivity",
                "p_any_emails": ["staging@company.io"],
                "p_any_ip_addresses": ["1.2.3.4"],
                "p_event_time": "2023-04-25 19:20:42.138",
                "p_log_type": "GCP.AuditLog",
                "p_parse_time": "2023-04-25 19:22:14.743",
                "p_row_id": "b2e9b7f5dc85a69981fac2e417b7bb03",
                "p_schema_version": 0,
                "p_source_id": "5b77391b-afad-46c7-8ddc-b8e21d4726b3",
                "p_source_label": "gcplogsource2",
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "staging@company.io"},
                    "authorizationInfo": [
                        {
                            "granted": True,
                            "permission": "iap.webServices.setIamPolicy",
                            "resourceAttributes": {
                                "name": "projects/123456789012/iap_web/compute/services/7312383563505470445",
                                "service": "iap.googleapis.com",
                                "type": "iap.googleapis.com/WebService",
                            },
                        },
                    ],
                    "methodName": "google.cloud.iap.v1.IdentityAwareProxyAdminService.SetIamPolicy",
                    "request": {
                        "@type": "type.googleapis.com/google.iam.v1.SetIamPolicyRequest",
                        "policy": {
                            "bindings": [
                                {
                                    "members": ["serviceAccount:test-account3@gcp-project1.iam.gserviceaccount.com"],
                                    "role": "roles/viewer",
                                },
                            ],
                            "etag": "ACAB",
                        },
                        "resource": "projects/123456789012/iap_web/compute/services/7312383563505470445",
                    },
                    "requestMetadata": {
                        "callerIP": "1.2.3.4",
                        "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/112.0.0.0 Safari/537.36,gzip(gfe),gzip(gfe)",
                        "destinationAttributes": {},
                        "requestAttributes": {"auth": {}, "time": "2023-04-25T19:20:42.399215146Z"},
                    },
                    "resourceName": "projects/123456789012/iap_web/compute/services/7312383563505470445",
                    "response": {
                        "@type": "type.googleapis.com/google.iam.v1.Policy",
                        "bindings": [
                            {
                                "members": ["serviceAccount:test-account3@gcp-project1.iam.gserviceaccount.com"],
                                "role": "roles/viewer",
                            },
                        ],
                        "etag": "BwX6LgT4YMw=",
                    },
                    "serviceName": "iap.googleapis.com",
                },
                "receiveTimestamp": "2023-04-25 19:20:43.033",
                "resource": {
                    "labels": {"backend_service_id": "", "location": "", "project_id": "gcp-project1"},
                    "type": "gce_backend_service",
                },
                "severity": "NOTICE",
                "timestamp": "2023-04-25 19:20:42.138",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class PushSecurityNewAppDetected(Rule):
    id = "Push.Security.New.App.Detected-prototype"
    display_name = "Push Security New App Detected"
    log_types = [LogType.PUSH_SECURITY_ENTITIES]
    default_severity = Severity.INFO

    def rule(self, event):
        if event.get("object") != "APP":
            return False
        if event.get("type") == "CREATE":
            return True
        return False

    def title(self, event):
        new_type = event.deep_get("new", "type")
        return f"New app in use: {new_type}"

    tests = [
        RuleTest(
            name="New App",
            expected_result=True,
            log={
                "id": "c478966c-f927-411c-b919-179832d3d50c",
                "new": {
                    "approvalStatus": None,
                    "creationTimestamp": 1698064423.0,
                    "id": "2a2197de-ad2c-47e4-8dcb-fb0f04cf83e0",
                    "notes": "",
                    "ownerId": None,
                    "sensitivityLevel": None,
                    "type": "ZAPIER",
                },
                "object": "APP",
                "old": None,
                "timestamp": 1698604061.0,
                "type": "CREATE",
                "version": "1",
            },
        ),
        RuleTest(
            name="App Updated",
            expected_result=False,
            log={
                "id": "c478966c-f927-411c-b919-179832d3d50c",
                "new": {
                    "approvalStatus": "APPROVED",
                    "creationTimestamp": 1698064423.0,
                    "id": "2a2197de-ad2c-47e4-8dcb-fb0f04cf83e0",
                    "notes": "Last security audit: 16 January 2023.\n",
                    "ownerId": "87569da6-fb7a-4df7-8ce2-246c14044911",
                    "sensitivityLevel": "HIGH",
                    "type": "ZAPIER",
                },
                "object": "APP",
                "old": {
                    "approvalStatus": "UNDER_REVIEW",
                    "creationTimestamp": 1698064423.0,
                    "id": "2a2197de-ad2c-47e4-8dcb-fb0f04cf83e0",
                    "notes": "Initial submission for review.\n",
                    "ownerId": "87569da6-fb7a-4df7-8ce2-246c14044911",
                    "sensitivityLevel": "MEDIUM",
                    "type": "ZAPIER",
                },
                "timestamp": 1698604061.0,
                "type": "UPDATE",
                "version": "1",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class OktaSSOtoAWS(Rule):
    id = "Okta.SSO.to.AWS-prototype"
    display_name = "SIGNAL - Okta SSO to AWS"
    create_alert = False
    log_types = [LogType.OKTA_SYSTEM_LOG]
    default_severity = Severity.INFO

    def rule(self, event):
        return all(
            [
                event.get("eventType") == "user.authentication.sso",
                event.deep_get("outcome", "result") == "SUCCESS",
                "AWS IAM Identity Center" in event.deep_walk("target", "displayName", default=[]),
            ],
        )

    def alert_context(self, event):
        return {"actor": event.deep_get("actor", "alternateId", default="").split("@")[0]}

    tests = [
        RuleTest(
            name="AWS SSO via Okta",
            expected_result=True,
            log={
                "displayMessage": "User single sign on to app",
                "eventType": "user.authentication.sso",
                "legacyEventType": "app.auth.sso",
                "outcome": {"result": "SUCCESS"},
                "securityContext": {},
                "severity": "INFO",
                "target": [
                    {
                        "alternateId": "AWS Production",
                        "detailEntry": {"signOnModeType": "SAML_2_0"},
                        "displayName": "AWS IAM Identity Center",
                        "id": "0oaua5ldoougycQAO696",
                        "type": "AppInstance",
                    },
                    {"alternateId": "aardvark", "displayName": "aardvark", "id": "0ua8aardvarkD697", "type": "AppUser"},
                ],
                "transaction": {"detail": {}, "id": "1a3852fc0d172ecdad0e2447e47fbc98", "type": "WEB"},
                "uuid": "35cae732-21bd-11ef-a011-dd05aa53a11a",
                "version": "0",
            },
        ),
        RuleTest(
            name="AWS SSO via Okta without app name",
            expected_result=False,
            log={
                "displayMessage": "User single sign on to app",
                "eventType": "user.authentication.sso",
                "legacyEventType": "app.auth.sso",
                "outcome": {"result": "SUCCESS"},
                "securityContext": {},
                "severity": "INFO",
                "target": [{"alternateId": "aardvark", "id": "0ua8aardvarkD697", "type": "AppUser"}],
                "transaction": {"detail": {}, "id": "1a3852fc0d172ecdad0e2447e47fbc98", "type": "WEB"},
                "uuid": "35cae732-21bd-11ef-a011-dd05aa53a11a",
                "version": "0",
            },
        ),
    ]



import json

from panther_detection_helpers.caching import add_to_string_set

from pypanther import LogType, Rule, RuleMock, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_rule_context


@panther_managed
class AWSSecretsManagerRetrieveSecretsMultiRegion(Rule):
    id = "AWS.SecretsManager.RetrieveSecretsMultiRegion-prototype"
    display_name = "AWS Secrets Manager Retrieve Secrets Multi-Region"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Credential Access", "Stratus Red Team"]
    reports = {"MITRE ATT&CK": ["TA0006:T1552"]}
    default_severity = Severity.MEDIUM
    default_description = "An attacker attempted to retrieve a high number of Secrets Manager secrets by batch, through secretsmanager:BatchGetSecretValue (released Novemeber 2023).  An attacker may attempt to retrieve a high number of secrets by batch, to avoid detection and generate fewer calls. Note that the batch size is limited to 20 secrets. This rule identifies BatchGetSecretValue events for multiple regions in a short period of time.\n"
    default_runbook = "https://aws.amazon.com/blogs/security/how-to-use-the-batchgetsecretsvalue-api-to-improve-your-client-side-applications-with-aws-secrets-manager/"
    default_reference = "https://stratus-red-team.cloud/attack-techniques/AWS/aws.credential-access.secretsmanager-batch-retrieve-secrets/"
    dedup_period_minutes = 1440
    summary_attributes = ["eventName", "userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]
    RULE_ID = "AWS.SecretsManager.RetrieveSecretsMultiRegion"
    UNIQUE_REGION_THRESHOLD = 5
    WITHIN_TIMEFRAME_MINUTES = 10

    def rule(self, event):
        if event.get("eventName") != "BatchGetSecretValue":
            return False
        user = event.udm("actor_user")
        key = f"{self.RULE_ID}-{user}"
        unique_regions = add_to_string_set(key, event.get("awsRegion"), self.WITHIN_TIMEFRAME_MINUTES * 60)
        if isinstance(unique_regions, str):
            unique_regions = json.loads(unique_regions)
        if len(unique_regions) >= self.UNIQUE_REGION_THRESHOLD:
            return True
        return False

    def title(self, event):
        user = event.udm("actor_user")
        return f"[{user}] attempted to retrieve secrets from AWS Secrets Manager in multiple regions"

    def alert_context(self, event):
        return aws_rule_context(event)

    tests = [
        RuleTest(
            name="BatchGetSecretValue Catch-All",
            expected_result=True,
            mocks=[
                RuleMock(
                    object_name="add_to_string_set",
                    return_value='["us-east-1", "us-west-2", "eu-west-1", "ap-southeast-1", "ap-northeast-1"]',
                ),
            ],
            log={
                "eventSource": "secretsmanager.amazonaws.com",
                "eventName": "BatchGetSecretValue",
                "requestParameters": {"filters": [{"key": "tag-key", "values": ["!tagKeyThatWillNeverExist"]}]},
                "responseElements": None,
                "readOnly": True,
                "eventType": "AwsApiCall",
                "managementEvent": True,
                "recipientAccountId": "012345678901",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_cloudtrail_success, aws_rule_context


@panther_managed
class AWSIAMAttachAdminUserPolicy(Rule):
    id = "AWS.IAM.AttachAdminUserPolicy-prototype"
    display_name = "IAM User Policy Attached with Administrator Access"
    create_alert = False
    log_types = [LogType.AWS_CLOUDTRAIL]
    reports = {"CIS": ["1.1"], "MITRE ATT&CK": ["TA0007:T1078"]}
    default_severity = Severity.INFO
    default_description = (
        "An IAM user policy was attached with Administrator Access, which could indicate a potential security risk.\n"
    )
    default_runbook = (
        "Check if the user policy was attached by an authorized user. If not, investigate the user policy attachment."
    )
    default_reference = "https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies.html"

    def rule(self, event):
        if not aws_cloudtrail_success(event) or event.get("eventName") != "AttachUserPolicy":
            return False
        policy = event.deep_get("requestParameters", "policyArn", default="POLICY_NOT_FOUND")
        return policy.endswith("AdministratorAccess")

    def alert_context(self, event):
        context = aws_rule_context(event)
        context["request_username"] = event.deep_get("requestParameters", "userName", default="USERNAME_NOT_FOUND")
        return context

    tests = [
        RuleTest(
            name="IAM User Policy Attached with Administrator Access",
            expected_result=True,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "tester",
                    "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "sessionContext": {
                        "sessionIssuer": {
                            "type": "Role",
                            "principalId": "1111",
                            "arn": "arn:aws:iam::123456789012:role/tester",
                            "accountId": "123456789012",
                            "userName": "Tester",
                        },
                        "webIdFederationData": {},
                        "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                    },
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "iam.amazonaws.com",
                "eventName": "AttachUserPolicy",
                "awsRegion": "us-west-2",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "console.amazonaws.com",
                "requestParameters": {
                    "policyArn": "arn:aws:iam::aws:policy/AdministratorAccess",
                    "userName": "new-user",
                },
                "responseElements": None,
                "requestID": "1",
                "eventID": "1",
                "readOnly": False,
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
        RuleTest(
            name="IAM User Policy Attached without Administrator Access",
            expected_result=False,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "AssumedRole",
                    "principalId": "tester",
                    "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "sessionContext": {
                        "sessionIssuer": {
                            "type": "Role",
                            "principalId": "1111",
                            "arn": "arn:aws:iam::123456789012:role/tester",
                            "accountId": "123456789012",
                            "userName": "Tester",
                        },
                        "webIdFederationData": {},
                        "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                    },
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "iam.amazonaws.com",
                "eventName": "AttachUserPolicy",
                "awsRegion": "us-west-2",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "console.amazonaws.com",
                "requestParameters": {"policyArn": "arn:aws:iam::aws:policy/ReadOnlyAccess", "userName": "new-user"},
                "responseElements": None,
                "requestID": "1",
                "eventID": "1",
                "readOnly": False,
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.duo import deserialize_administrator_log_event_description


@panther_managed
class DUOAdminActionMarkedFraudulent(Rule):
    id = "DUO.Admin.Action.MarkedFraudulent-prototype"
    display_name = "Duo Admin Marked Push Fraudulent"
    dedup_period_minutes = 15
    log_types = [LogType.DUO_ADMINISTRATOR]
    tags = ["Duo"]
    default_severity = Severity.MEDIUM
    default_description = "A Duo push was marked fraudulent by an admin."
    default_reference = "https://duo.com/docs/adminapi#administrator-logs"
    default_runbook = "Follow up with the administrator to determine reasoning for marking fraud."

    def rule(self, event):
        event_description = deserialize_administrator_log_event_description(event)
        return event.get("action") == "admin_2fa_error" and "fraudulent" in event_description.get("error", "").lower()

    def title(self, event):
        event_description = deserialize_administrator_log_event_description(event)
        admin_username = event.get("username", "Unknown")
        user_email = event_description.get("email", "Unknown")
        return f"Duo Admin [{admin_username}] denied due to an anomalous 2FA push for [{user_email}]"

    def alert_context(self, event):
        event_description = deserialize_administrator_log_event_description(event)
        return {
            "reason": event_description.get("error", ""),
            "reporting_admin": event.get("username", ""),
            "user": event_description.get("email", ""),
            "ip_address": event_description.get("ip_address", ""),
        }

    tests = [
        RuleTest(
            name="marked_fraud",
            expected_result=True,
            log={
                "action": "admin_2fa_error",
                "description": '{"ip_address": "12.12.12.12", "email": "example@example.io", "factor": "push", "error": "Login request reported as fraudulent."}',
                "isotimestamp": "2022-12-14 20:11:53",
                "timestamp": "2022-12-14 20:11:53",
                "username": "John P. Admin",
            },
        ),
        RuleTest(
            name="different_admin_action",
            expected_result=False,
            log={
                "action": "admin_update",
                "description": "{}",
                "isotimestamp": "2022-12-14 20:11:53",
                "timestamp": "2022-12-14 20:11:53",
                "username": "John P. Admin",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_cloudtrail_success, aws_rule_context


@panther_managed
class AWSBedrockDeleteModelInvocationLoggingConfiguration(Rule):
    id = "AWS.Bedrock.DeleteModelInvocationLoggingConfiguration-prototype"
    display_name = "AWS Bedrock Model Invocation Logging Configuration Deleted"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Bedrock", "Impair Defenses: Impair Command History Logging", "Defense Evastion"]
    default_severity = Severity.MEDIUM
    reports = {"MITRE ATT&CK": ["TA0005:T1562.003"]}
    default_description = "An Amazon Bedrock Model Invocation Logging Configuration was deleted. Use model invocation logging to collect metadata, requests, and responses for all model invocations in your account. Deleting a model invocation logging configuration can have security implications to your AI workloads.\n"
    default_runbook = "Review the model invocation logging configuration deletion to ensure that it was authorized and that it does not introduce security risks to your AI workloads. If the model invocation logging configuration deletion was unauthorized, investigate the incident and take appropriate action.\n"
    default_reference = "https://docs.aws.amazon.com/bedrock/latest/userguide/model-invocation-logging.html"
    summary_attributes = ["userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]

    def rule(self, event):
        if (
            event.get("eventSource") == "bedrock.amazonaws.com"
            and event.get("eventName") == "DeleteModelInvocationLoggingConfiguration"
            and aws_cloudtrail_success(event)
        ):
            return True
        return False

    def title(self, event):
        user = event.udm("actor_user")
        return f"User [{user}] deleted Bedrock model invocation logging configuration"

    def alert_context(self, event):
        return aws_rule_context(event)

    tests = [
        RuleTest(
            name="Model Invocation Logging Configuration Deleted",
            expected_result=True,
            log={
                "awsRegion": "us-west-2",
                "eventCategory": "Management",
                "eventID": "28773860-a4fd-47c7-a215-6f0e6e6e532f",
                "eventName": "DeleteModelInvocationLoggingConfiguration",
                "eventSource": "bedrock.amazonaws.com",
                "eventTime": "2025-01-21 17:49:47.000000000",
                "eventType": "AwsApiCall",
                "eventVersion": "1.09",
                "managementEvent": True,
                "readOnly": False,
                "recipientAccountId": "123123123123",
                "requestID": "7b9b25ca-be2d-4428-9793-0a677c32b823",
                "sessionCredentialFromConsole": True,
                "sourceIPAddress": "161.97.249.211",
                "tlsDetails": {
                    "cipherSuite": "TLS_AES_128_GCM_SHA256",
                    "clientProvidedHostHeader": "bedrock.us-west-2.amazonaws.com",
                    "tlsVersion": "TLSv1.3",
                },
                "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/131.0.0.0 Safari/537.36",
                "userIdentity": {
                    "accessKeyId": "ASIAQWERQWERQWERQWER",
                    "accountId": "123123123123",
                    "arn": "arn:aws:sts::123123123123:assumed-role/DevAdmin/dr.evil",
                    "principalId": "AROAQWERQWERQWERQWER:dr.evil",
                    "sessionContext": {
                        "attributes": {"creationDate": "2025-01-21T16:08:03Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "123123123123",
                            "arn": "arn:aws:iam::123123123123:role/aws-reserved/sso.amazonaws.com/us-west-2/DevAdmin",
                            "principalId": "AROAQWERQWERQWERQWER",
                            "type": "Role",
                            "userName": "DevAdmin",
                        },
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="List Guardrails",
            expected_result=False,
            log={
                "awsRegion": "us-west-2",
                "eventCategory": "Management",
                "eventID": "906c4056-df1e-4565-a40b-2ba216a0c849",
                "eventName": "ListGuardrails",
                "eventSource": "bedrock.amazonaws.com",
                "eventTime": "2025-01-21 18:12:33.000000000",
                "eventType": "AwsApiCall",
                "eventVersion": "1.09",
                "managementEvent": True,
                "readOnly": True,
                "recipientAccountId": "123123123123",
                "requestID": "9219ab18-cddf-4376-afc6-cc4edf2c2f0f",
                "requestParameters": {"maxResults": 1000},
                "sessionCredentialFromConsole": True,
                "sourceIPAddress": "123.123.123.123",
                "tlsDetails": {
                    "cipherSuite": "TLS_AES_128_GCM_SHA256",
                    "clientProvidedHostHeader": "bedrock.us-west-2.amazonaws.com",
                    "tlsVersion": "TLSv1.3",
                },
                "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/131.0.0.0 Safari/537.36",
                "userIdentity": {
                    "accessKeyId": "ASIAQWERQWERQWERQWER",
                    "accountId": "123123123123",
                    "arn": "arn:aws:sts::123123123123:assumed-role/DevAdmin/dr.evil",
                    "principalId": "AROAQWERQWERQWERQWER:dr.evil",
                    "sessionContext": {
                        "attributes": {"creationDate": "2025-01-21T16:08:03Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "123123123123",
                            "arn": "arn:aws:iam::123123123123:role/aws-reserved/sso.amazonaws.com/us-west-2/DevAdmin",
                            "principalId": "AROAQWERQWERQWERQWER",
                            "type": "Role",
                            "userName": "DevAdmin",
                        },
                    },
                    "type": "AssumedRole",
                },
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.wiz import wiz_actor, wiz_alert_context, wiz_success


@panther_managed
class WizRevokeUserSessions(Rule):
    id = "Wiz.Revoke.User.Sessions-prototype"
    default_description = "This rule detects user sessions revoked."
    display_name = "Wiz Revoke User Sessions"
    default_runbook = (
        "Verify that this change was planned. If not, revoke all the sessions of the account and change its credentials"
    )
    default_reference = (
        "https://www.wiz.io/blog/storm-0558-compromised-microsoft-key-enables-authentication-of-countless-micr"
    )
    default_severity = Severity.MEDIUM
    reports = {"MITRE ATT&CK": ["TA0040:T1531"]}
    log_types = [LogType.WIZ_AUDIT]

    def rule(self, event):
        if not wiz_success(event):
            return False
        return event.get("action", "ACTION_NOT_FOUND") == "RevokeUserSessions"

    def title(self, event):
        actor = wiz_actor(event)
        return f"[Wiz]: [{event.get('action', 'ACTION_NOT_FOUND')}] action performed by {actor.get('type')} [{actor.get('name')}]"

    def dedup(self, event):
        return event.get("id")

    def alert_context(self, event):
        return wiz_alert_context(event)

    tests = [
        RuleTest(
            name="RevokeUserSessions",
            expected_result=True,
            log={
                "id": "07fdb41e-e83d-46e2-814a-6cebc47acf97",
                "action": "RevokeUserSessions",
                "requestId": "5fa96b8f-2c85-4c2d-b0f9-d4a4307ea8a7",
                "status": "SUCCESS",
                "timestamp": "2024-07-31T17:55:29.239928Z",
                "actionParameters": {"input": {"id": "<redacted>"}, "selection": ["__typename", "_stub"]},
                "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36",
                "sourceIP": "12.34.56.78",
                "serviceAccount": None,
                "user": {"id": "test.user@company.com", "name": "user@company.com"},
            },
        ),
        RuleTest(
            name="CreateUser",
            expected_result=False,
            log={
                "id": "220d23be-f07c-4d97-b4a6-87ad04eddb14",
                "action": "CreateUser",
                "requestId": "0d9521b2-c3f8-4a73-bf7c-20257788752e",
                "status": "SUCCESS",
                "timestamp": "2024-07-29T09:40:15.66643Z",
                "actionParameters": {
                    "input": {
                        "assignedProjectIds": None,
                        "email": "testy@company.com",
                        "expiresAt": None,
                        "name": "Test User",
                        "role": "GLOBAL_ADMIN",
                    },
                    "selection": ["__typename", {"user": ["__typename", "id"]}],
                },
                "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36",
                "sourceIP": "8.8.8.8",
                "serviceAccount": None,
                "user": {"id": "someuser@company.com", "name": "someuser@company.com"},
            },
        ),
        RuleTest(
            name="RevokeUserSessions - Fail",
            expected_result=False,
            log={
                "id": "07fdb41e-e83d-46e2-814a-6cebc47acf97",
                "action": "RevokeUserSessions",
                "requestId": "5fa96b8f-2c85-4c2d-b0f9-d4a4307ea8a7",
                "status": "FAILED",
                "timestamp": "2024-07-31T17:55:29.239928Z",
                "actionParameters": {},
                "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36",
                "sourceIP": "12.34.56.78",
                "serviceAccount": None,
                "user": {"id": "test.user@company.com", "name": "user@company.com"},
            },
        ),
    ]



from ipaddress import ip_address

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_rule_context


@panther_managed
class AWSCloudTrailUnauthorizedAPICall(Rule):
    id = "AWS.CloudTrail.UnauthorizedAPICall-prototype"
    display_name = "Monitor Unauthorized API Calls"
    dedup_period_minutes = 1440
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Discovery:Cloud Service Discovery"]
    reports = {"CIS": ["3.1"], "MITRE ATT&CK": ["TA0007:T1526"]}
    default_severity = Severity.INFO
    create_alert = False
    default_description = "An unauthorized AWS API call was made"
    default_runbook = "https://docs.runpanther.io/alert-runbooks/built-in-rules/aws-unauthorized-api-call"
    default_reference = "https://amzn.to/3aOukaA"
    summary_attributes = ["eventName", "userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]
    threshold = 20
    # Do not alert on these access denied errors for these events.
    # Events could be exceptions because they are particularly noisy and provide little to no value,
    # or because they are expected as part of the normal operating procedure for certain tools.
    # Noisy, doesn't really provide any actionable info
    # The audit role hits this when scanning locked down resources
    EVENT_EXCEPTIONS = {"DescribeEventAggregates", "ListResourceTags"}

    def rule(self, event):
        # Validate the request came from outside of AWS
        try:
            ip_address(event.get("sourceIPAddress"))
        except ValueError:
            return False
        return event.get("errorCode") == "AccessDenied" and event.get("eventName") not in self.EVENT_EXCEPTIONS

    def dedup(self, event):
        return event.deep_get("userIdentity", "principalId", default="<UNKNOWN_PRINCIPAL>")

    def title(self, event):
        return f"Access denied to {event.deep_get('userIdentity', 'type')} [{self.dedup(event)}]"

    def alert_context(self, event):
        return aws_rule_context(event)

    tests = [
        RuleTest(
            name="Unauthorized API Call from Within AWS (IP)",
            expected_result=True,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "IAMUser",
                    "principalId": "1111",
                    "arn": "arn:aws:iam::123456789012:user/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "userName": "tester",
                    "sessionContext": {
                        "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                    },
                    "invokedBy": "signin.amazonaws.com",
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "iam.amazonaws.com",
                "eventName": "CreateServiceLinkedRole",
                "awsRegion": "us-east-1",
                "sourceIPAddress": "3.10.107.144",
                "errorCode": "AccessDenied",
                "errorMessage": "User: arn:aws:iam::123456789012:user/tester is not authorized to perform: iam:Action on resource: arn:aws:iam::123456789012:resource",
                "userAgent": "sqs.amazonaws.com",
                "requestParameters": None,
                "responseElements": None,
                "requestID": "1",
                "eventID": "1",
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
        RuleTest(
            name="Unauthorized API Call from Within AWS (FQDN)",
            expected_result=False,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "IAMUser",
                    "principalId": "1111",
                    "arn": "arn:aws:iam::123456789012:user/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "userName": "tester",
                    "sessionContext": {
                        "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                    },
                    "invokedBy": "signin.amazonaws.com",
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "iam.amazonaws.com",
                "eventName": "CreateServiceLinkedRole",
                "awsRegion": "us-east-1",
                "sourceIPAddress": "sqs.amazonaws.com",
                "errorCode": "AccessDenied",
                "errorMessage": "User: arn:aws:iam::123456789012:user/tester is not authorized to perform: iam:Action on resource: arn:aws:iam::123456789012:resource",
                "userAgent": "sqs.amazonaws.com",
                "requestParameters": None,
                "responseElements": None,
                "requestID": "1",
                "eventID": "1",
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
        RuleTest(
            name="Authorized API Call",
            expected_result=False,
            log={
                "eventVersion": "1.05",
                "userIdentity": {
                    "type": "IAMUser",
                    "principalId": "1111",
                    "arn": "arn:aws:iam::123456789012:user/tester",
                    "accountId": "123456789012",
                    "accessKeyId": "1",
                    "userName": "tester",
                    "sessionContext": {
                        "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                    },
                    "invokedBy": "signin.amazonaws.com",
                },
                "eventTime": "2019-01-01T00:00:00Z",
                "eventSource": "iam.amazonaws.com",
                "eventName": "CreateServiceLinkedRole",
                "awsRegion": "us-east-1",
                "sourceIPAddress": "111.111.111.111",
                "userAgent": "signin.amazonaws.com",
                "requestParameters": None,
                "responseElements": None,
                "requestID": "1",
                "eventID": "1",
                "eventType": "AwsApiCall",
                "recipientAccountId": "123456789012",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.crowdstrike_event_streams import audit_keys_dict, cs_alert_context


@panther_managed
class CrowdstrikeAllowlistRemoved(Rule):
    id = "Crowdstrike.AllowlistRemoved-prototype"
    display_name = "Crowdstrike Allowlist Removed"
    log_types = [LogType.CROWDSTRIKE_EVENT_STREAMS]
    default_severity = Severity.MEDIUM
    reports = {"MITRE ATT&CK": ["TA0040:T1531"]}
    default_description = "A user deleted an allowlist"
    default_runbook = "Confirm if the deleted allowlist is needed."

    def rule(self, event):
        # Return True if allowlist is deleted
        if event.deep_get("event", "OperationName") == "DeleteAllowlistGroup":
            return True
        # Return True if allowlist is disabled
        if event.deep_get("event", "OperationName") == "UpdateAllowlistGroup":
            audit_keys = audit_keys_dict(event)
            return audit_keys.get("active") == "false" and audit_keys.get("old_active") == "true"
        return False

    def title(self, event):
        actor = event.deep_get("event", "UserId")
        audit_keys = audit_keys_dict(event)
        list_name = audit_keys.get("group_name", "UNKNOWN_GROUP")
        verb = {"DeleteAllowlistGroup": "deleted", "UpdateAllowlistGroup": "disabled"}.get(
            event.deep_get("event", "OperationName"),
            "removed",
        )
        return f'{actor} {verb} IP allowlist "{list_name}"'

    def dedup(self, event):
        # We wanna group alerts if a user disables, then deletes the same allowlist
        actor = event.deep_get("event", "UserId")
        audit_keys = audit_keys_dict(event)
        list_name = audit_keys.get("group_name", "UNKNOWN_GROUP")
        return f"{actor}-{list_name}"

    def alert_context(self, event):
        return cs_alert_context(event)

    def severity(self, event):
        # Downgrade severity if a disabled allowlist was deleted
        if all(
            [
                event.deep_get("event", "OperationName") == "DeleteAllowlistGroup",
                audit_keys_dict(event).get("enabled") == "false",
            ],
        ):
            return "INFO"
        return "DEFAULT"

    tests = [
        RuleTest(
            name="Enabled Allow List Deleted",
            expected_result=True,
            log={
                "event": {
                    "AuditKeyValues": [
                        {"Key": "cidrs", "ValueString": "[0.0.0.0/8]"},
                        {"Key": "contexts", "ValueString": "[API]"},
                        {"Key": "active", "ValueString": "true"},
                        {"Key": "allowlist_group_id", "ValueString": "782f842e-98dd-4ee7-9793-33abf8647656"},
                        {"Key": "group_name", "ValueString": "my_allow_list"},
                        {"Key": "description", "ValueString": ""},
                    ],
                    "OperationName": "DeleteAllowlistGroup",
                    "ServiceName": "Crowdstrike Allowlist Management",
                    "Success": True,
                    "UTCTimestamp": "2024-07-26 19:43:35.000000000",
                    "UserId": "wormtongue@isengard.org",
                    "UserIp": "1.2.3.4",
                },
                "metadata": {
                    "customerIDString": "fake_customer_id",
                    "eventCreationTime": "2024-07-26 19:43:35.082000000",
                    "eventType": "AuthActivityAuditEvent",
                    "offset": 366125,
                    "version": "1.0",
                },
            },
        ),
        RuleTest(
            name="Disabled Allow List Deleted",
            expected_result=True,
            log={
                "event": {
                    "AuditKeyValues": [
                        {"Key": "cidrs", "ValueString": "[0.0.0.0/8]"},
                        {"Key": "contexts", "ValueString": "[API]"},
                        {"Key": "active", "ValueString": "false"},
                        {"Key": "allowlist_group_id", "ValueString": "782f842e-98dd-4ee7-9793-33abf8647656"},
                        {"Key": "group_name", "ValueString": "my_allow_list"},
                        {"Key": "description", "ValueString": ""},
                    ],
                    "OperationName": "DeleteAllowlistGroup",
                    "ServiceName": "Crowdstrike Allowlist Management",
                    "Success": True,
                    "UTCTimestamp": "2024-07-26 19:43:35.000000000",
                    "UserId": "wormtongue@isengard.org",
                    "UserIp": "1.2.3.4",
                },
                "metadata": {
                    "customerIDString": "fake_customer_id",
                    "eventCreationTime": "2024-07-26 19:43:35.082000000",
                    "eventType": "AuthActivityAuditEvent",
                    "offset": 366125,
                    "version": "1.0",
                },
            },
        ),
        RuleTest(
            name="Allowlist Disabled",
            expected_result=True,
            log={
                "event": {
                    "AuditKeyValues": [
                        {"Key": "old_active", "ValueString": "true"},
                        {"Key": "group_name", "ValueString": "my_allow_list"},
                        {"Key": "old_group_name", "ValueString": "b"},
                        {"Key": "cidrs", "ValueString": "[1.2.3.4/8]"},
                        {"Key": "contexts", "ValueString": "[API UI]"},
                        {"Key": "active", "ValueString": "false"},
                        {"Key": "old_allowlist_group_id", "ValueString": "24821376-7e77-431e-9469-74846978fe64"},
                        {"Key": "old_description", "ValueString": ""},
                        {"Key": "old_cidrs", "ValueString": "[1.2.3.4/8]"},
                        {"Key": "allowlist_group_id", "ValueString": "24821376-7e77-431e-9469-74846978fe64"},
                        {"Key": "description", "ValueString": ""},
                        {"Key": "old_contexts", "ValueString": "[API UI]"},
                    ],
                    "OperationName": "UpdateAllowlistGroup",
                    "ServiceName": "Crowdstrike Allowlist Management",
                    "Success": True,
                    "UTCTimestamp": "2024-07-26 19:52:14.000000000",
                    "UserId": "wormtongue@isengard.org",
                    "UserIp": "1.2.3.4",
                },
                "metadata": {
                    "customerIDString": "fake_customer_id",
                    "eventCreationTime": "2024-07-26 19:52:14.438000000",
                    "eventType": "AuthActivityAuditEvent",
                    "offset": 366171,
                    "version": "1.0",
                },
            },
        ),
        RuleTest(
            name="Unrelated Event",
            expected_result=False,
            log={
                "metadata": {
                    "customerIDString": "face_customer_id",
                    "offset": 1238741,
                    "eventType": "AuthActivityAuditEvent",
                    "eventCreationTime": "2024-07-22 15:50:16.923000000",
                    "version": "1.0",
                },
                "event": {
                    "UserId": "bilbo.baggins@hobbiton.co",
                    "UserIp": "1.1.1.1",
                    "OperationName": "createUser",
                    "ServiceName": "CrowdStrike Authentication",
                    "Success": True,
                    "UTCTimestamp": "2024-07-22 15:50:16.923000000",
                    "AuditKeyValues": [{"Key": "target_name", "ValueString": "frodo.baggins@hobbiton.co"}],
                },
            },
        ),
    ]



"""
Used to define global helpers and variables for AWS events.
"""

import base64
import binascii
import os
from typing import Any, Dict, List

import boto3

from pypanther.helpers.base import pantherflow_investigation
from pypanther.helpers.config import config


class BadLookup(Exception):
    """Error returned when a resource lookup fails."""


class PantherBadInput(Exception):
    """Error returned when a Panther helper function is provided bad input."""


AWS_ACCOUNTS = config.AWS_ACCOUNTS
_RESOURCE_TABLE = None  # boto3.Table resource, lazily constructed
FIPS_ENABLED = os.getenv("ENABLE_FIPS", "").lower() == "true"
FIPS_SUFFIX = "-fips." + os.getenv("AWS_REGION", "") + ".amazonaws.com"


def aws_strip_role_session_id(user_identity_arn):
    # The ARN structure is arn:aws:sts::123456789012:assumed-role/RoleName/<sessionId>
    arn_parts = user_identity_arn.split("/")
    if arn_parts:
        return "/".join(arn_parts[:2])
    return user_identity_arn


def aws_rule_context(event):
    return {
        "eventName": event.get("eventName", "<MISSING_EVENT_NAME>"),
        "eventSource": event.get("eventSource", "<MISSING_ACCOUNT_ID>"),
        "awsRegion": event.get("awsRegion", "<MISSING_AWS_REGION>"),
        "recipientAccountId": event.get("recipientAccountId", "<MISSING_ACCOUNT_ID>"),
        "sourceIPAddress": event.get("sourceIPAddress", "<MISSING_SOURCE_IP>"),
        "userAgent": event.get("userAgent", "<MISSING_USER_AGENT>"),
        "userIdentity": event.get("userIdentity", "<MISSING_USER_IDENTITY>"),
        "PantherFlow Investigation": pantherflow_investigation(event),
    }


def aws_guardduty_context(event):
    return {
        "description": event.get("description", "<MISSING DESCRIPTION>"),
        "severity": event.get("severity", "<MISSING SEVERITY>"),
        "id": event.get("id", "<MISSING ID>"),
        "type": event.get("type", "<MISSING TYPE>"),
        "resource": event.get("resource", {}),
        "service": event.get("service", {}),
        "accountId": event.get("accountId", "<MISSING ACCOUNT ID>"),
    }


def eks_panther_obj_ref(event):
    user = event.deep_get("user", "username", default="<NO_USERNAME>")
    source_ips = event.get("sourceIPs", ["0.0.0.0"])  # nosec
    verb = event.get("verb", "<NO_VERB>")
    obj_name = event.deep_get("objectRef", "name", default="<NO_OBJECT_NAME>")
    obj_ns = event.deep_get("objectRef", "namespace", default="<NO_OBJECT_NAMESPACE>")
    obj_res = event.deep_get("objectRef", "resource", default="<NO_OBJECT_RESOURCE>")
    obj_subres = event.deep_get("objectRef", "subresource", default="")
    p_source_label = event.get("p_source_label", "<NO_P_SOURCE_LABEL>")
    if obj_subres:
        obj_res = "/".join([obj_res, obj_subres])
    return {
        "actor": user,
        "ns": obj_ns,
        "object": obj_name,
        "resource": obj_res,
        "sourceIPs": source_ips,
        "verb": verb,
        "p_source_label": p_source_label,
    }


def aws_cloudtrail_success(event):
    if event.get("errorCode", "") or event.get("errorMessage", ""):
        return False
    return True


def aws_event_tense(event_name):
    """
    Convert an AWS CloudTrail eventName to be interpolated in alert titles

    An example is passing in StartInstance and returning 'started'.
    This would then be used in an alert title such as
    'The EC2 instance my-instance was started'.

    Args:
    ----
        event_name (str): The CloudTrail eventName

    Returns:
    -------
        str: A tensed version of the event name

    """
    mapping = {
        "Create": "created",
        "Delete": "deleted",
        "Start": "started",
        "Stop": "stopped",
        "Update": "updated",
    }
    for event_prefix, tensed in mapping.items():
        if event_name.startswith(event_prefix):
            return tensed
    # If the event pattern doesn't exist, return original
    return event_name


# Adapted from https://medium.com/@TalBeerySec/a-short-note-on-aws-key-id-f88cc4317489
def aws_key_account_id(aws_key: str):
    """Retrieve the AWS account ID associated with a given access key ID"""
    key_no_prefix = aws_key[4:]  # remove the four-character prefix
    base32_key = base64.b32decode(key_no_prefix)  # remainder of the key is base32-encoded
    decoded_key = base32_key[0:6]  # retrieve the 10-byte string

    # Convert the 10-byte string to an integer
    key_id_int = int.from_bytes(decoded_key, byteorder="big", signed=False)
    mask = int.from_bytes(binascii.unhexlify(b"7fffffffff80"), byteorder="big", signed=False)

    # Do a bitwise AND with the mask to retrieve the account ID
    # (divide the 10-byte key integer by 128 and remove the fractional part(s))
    account_id = (key_id_int & mask) >> 7
    return str(account_id)


def aws_regions() -> List[str]:
    """Return a list of AWS regions"""
    return [
        "ap-east-1",
        "ap-northeast-1",
        "ap-northeast-1",
        "ap-northeast-2",
        "ap-northeast-3",
        "ap-south-1",
        "ap-south-2",
        "ap-southeast-1",
        "ap-southeast-2",
        "ap-southeast-3",
        "ap-southeast-4",
        "ca-central-1",
        "ca-central-1",
        "eu-central-1",
        "eu-central-1",
        "eu-central-2",
        "eu-north-1",
        "eu-north-1",
        "eu-south-1",
        "eu-south-2",
        "eu-west-1",
        "eu-west-1",
        "eu-west-2",
        "eu-west-2",
        "eu-west-3",
        "eu-west-3",
        "il-central-1",
        "me-central-1",
        "me-south-1",
        "sa-east-1",
        "us-east-1",
        "us-east-2",
        "us-west-1",
        "us-west-2",
    ]


def lookup_aws_account_name(account_id):
    """
    Lookup the AWS account name, return the ID if not found

    Args:
    ----
        account_id (str): The AWS account ID

    Returns:
    -------
        str: The name of the AWS account ID
        or
        str: The AWS account ID (unnamed account)

    """
    return AWS_ACCOUNTS.get(account_id, f"{account_id} (unnamed account)")


def get_s3_arn_by_name(name: str) -> str:
    """This function is used to construct an s3 bucket ARN from its name."""
    if name == "":
        raise PantherBadInput("s3 name cannot be blank")
    return "arn:aws:s3:::" + name


def s3_lookup_by_name(name: str) -> Dict[str, Any]:
    """This function is used to get an S3 bucket resource from just its name."""
    return resource_lookup(get_s3_arn_by_name(name))


def resource_table() -> boto3.resource:
    """Lazily build resource table"""
    # pylint: disable=global-statement
    global _RESOURCE_TABLE
    if not _RESOURCE_TABLE:
        # pylint: disable=no-member
        _RESOURCE_TABLE = boto3.resource(
            "dynamodb",
            endpoint_url="https://dynamodb" + FIPS_SUFFIX if FIPS_ENABLED else None,
        ).Table("panther-resources")
    return _RESOURCE_TABLE


def resource_lookup(resource_id: str) -> Dict[str, Any]:
    """This function is used to get a resource from the resources-api based on its resourceID."""
    # Validate input so we can provide meaningful error messages to users
    if resource_id == "":
        raise PantherBadInput("resourceId cannot be blank")

    # Get the item from dynamo
    response = resource_table().get_item(Key={"id": resource_id})

    # Check if dynamo failed
    status_code = response["ResponseMetadata"]["HTTPStatusCode"]
    if status_code != 200:
        raise BadLookup("dynamodb - " + str(status_code) + " HTTPStatusCode")

    # Check if the item was found
    if "Item" not in response:
        raise BadLookup(resource_id + " not found")

    # Return just the attributes of the item
    return response["Item"]["attributes"]



from pypanther.rules.thinkstcanary.thinkst_canary_dcrc import ThinkstCanaryDCRC as ThinkstCanaryDCRC
from pypanther.rules.thinkstcanary.thinkst_canary_incident import ThinkstCanaryIncident as ThinkstCanaryIncident
from pypanther.rules.thinkstcanary.thinkst_canarytoken_incident import (
    ThinkstCanaryTokenIncident as ThinkstCanaryTokenIncident,
)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.gcp import gcp_alert_context


@panther_managed
class GCPComputeSnapshotUnexpectedDomain(Rule):
    display_name = "GCP Snapshot Creation Detection"
    enabled = False
    id = "GCP.Compute.Snapshot.UnexpectedDomain-prototype"
    default_severity = Severity.MEDIUM
    log_types = [LogType.GCP_AUDIT_LOG]
    tags = ["Configuration Required"]
    default_description = (
        "This rule detects when someone with an unexpected email domain creates a snapshot of a Compute Disk.\n"
    )
    default_runbook = "Investigate the snapshot creation to ensure it was authorized. Unauthorized snapshot creation can lead to data exfiltration.\n"
    default_reference = "https://cloud.google.com/compute/docs/disks/snapshots"
    EXPECTED_DOMAIN = "@your-domain.tld"

    def rule(self, event):
        if event.deep_get("protoPayload", "response", "error"):
            return False
        method = event.deep_get("protoPayload", "methodName", default="METHOD_NOT_FOUND")
        if method != "v1.compute.snapshots.insert":
            return False
        email = event.deep_get("protoPayload", "authenticationInfo", "principalEmail", default="")
        if not email.endswith(self.EXPECTED_DOMAIN):
            return True
        return False

    def title(self, event):
        actor = event.deep_get("protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        project = event.deep_get("resource", "labels", "project_id", default="<PROJECT_NOT_FOUND>")
        return f"[GCP]: Unexpected domain [{actor}] created a snapshot on project [{project}]"

    def alert_context(self, event):
        return gcp_alert_context(event)

    tests = [
        RuleTest(
            name="Snapshot creation by user with unexpected domain",
            expected_result=True,
            log={
                "insertId": "1abcd23efg456",
                "logName": "projects/test-project/logs/cloudaudit.googleapis.com%2Factivity",
                "protoPayload": {
                    "@type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "user@unexpected-domain.com"},
                    "methodName": "v1.compute.snapshots.insert",
                    "resourceName": "projects/test-project/global/snapshots/snapshot-1",
                    "serviceName": "compute.googleapis.com",
                },
                "resource": {"labels": {"project_id": "test-project"}, "type": "gce_snapshot"},
                "severity": "NOTICE",
                "timestamp": "2023-10-01T12:34:56.789Z",
            },
        ),
        RuleTest(
            name="Snapshot creation by user with expected domain",
            expected_result=False,
            log={
                "insertId": "2hijk34lmn789",
                "logName": "projects/test-project/logs/cloudaudit.googleapis.com%2Factivity",
                "protoPayload": {
                    "@type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "user@your-domain.tld"},
                    "methodName": "v1.compute.snapshots.insert",
                    "resourceName": "projects/test-project/global/snapshots/snapshot-2",
                    "serviceName": "compute.googleapis.com",
                },
                "resource": {"labels": {"project_id": "test-project"}, "type": "gce_snapshot"},
                "severity": "NOTICE",
                "timestamp": "2023-10-01T12:45:56.789Z",
            },
        ),
    ]



import ipaddress

from pypanther.base import DataModel, DataModelMapping, LogType
from pypanther.helpers import event_type
from pypanther.helpers.base import deep_get


def get_event_type(event):
    # currently, only tracking a few event types
    if event.get("eventName") == "ConsoleLogin" and deep_get(event, "userIdentity", "type") == "IAMUser":
        if deep_get(event, "responseElements", "ConsoleLogin") == "Failure":
            return event_type.FAILED_LOGIN
        if deep_get(event, "responseElements", "ConsoleLogin") == "Success":
            return event_type.SUCCESSFUL_LOGIN
    if event.get("eventName") == "CreateUser":
        return event_type.USER_ACCOUNT_CREATED
    if event.get("eventName") == "CreateAccountResult":
        return event_type.ACCOUNT_CREATED
    return None


def load_ip_address(event):
    """
    CloudTrail occasionally sets non-IPs in the sourceIPAddress field.
    This method ensures that either an IPv4 or IPv6 address is always returned.
    """
    source_ip = event.get("sourceIPAddress")
    if not source_ip:
        return None
    try:
        ipaddress.IPv4Address(source_ip)
    except ipaddress.AddressValueError:
        try:
            ipaddress.IPv6Address(source_ip)
        except ipaddress.AddressValueError:
            return None
    return source_ip


# get actor user from correct field based on identity type
# https://docs.aws.amazon.com/awscloudtrail/latest/userguide/cloudtrail-event-reference-user-identity.html#cloudtrail-event-reference-user-identity-fields
def get_actor_user(event):
    user_type = deep_get(event, "userIdentity", "type")
    if user_type == "Root":
        actor_user = deep_get(
            event,
            "userIdentity",
            "userName",
            default=deep_get(event, "userIdentity", "accountId", default="UnknownRootUser"),
        )
    elif user_type in ("IAMUser", "Directory", "Unknown", "SAMLUser", "WebIdentityUser"):
        actor_user = deep_get(event, "userIdentity", "userName", default=f"Unknown{user_type}")
    elif user_type in ("AssumedRole", "Role", "FederatedUser"):
        actor_user = deep_get(
            event,
            "userIdentity",
            "sessionContext",
            "sessionIssuer",
            "userName",
            default=f"Unknown{user_type}",
        )
    elif user_type == "IdentityCenterUser":
        actor_user = deep_get(
            event,
            "additionalEventData",
            "UserName",
            default=f"Unknown{user_type}",
        )
    elif user_type in ("AWSService", "AWSAccount"):
        actor_user = event.get("sourceIdentity", f"Unknown{user_type}")
    elif event.get("eventType") == "AwsServiceEvent":
        actor_user = deep_get(event, "userIdentity", "invokedBy", default="UnknownAwsServiceEvent")
    else:
        actor_user = "UnknownUser"
    return actor_user


class StandardAWSCloudTrail(DataModel):
    id: str = "Standard.AWS.CloudTrail"
    display_name: str = "AWS CloudTrail"
    enabled: bool = True
    log_types: list[str] = [LogType.AWS_CLOUDTRAIL]
    mappings: list[DataModelMapping] = [
        DataModelMapping(name="actor_user", method=get_actor_user),
        DataModelMapping(name="event_type", method=get_event_type),
        DataModelMapping(name="source_ip", method=load_ip_address),
        DataModelMapping(name="user_agent", path="userAgent"),
        DataModelMapping(name="user", path="$.responseElements.user.userName"),
        DataModelMapping(name="user_account_id", path="$.responseElements.user.userId"),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class GCPCloudStorageBucketsModifiedOrDeleted(Rule):
    default_description = "Detects GCP cloud storage bucket updates and deletes."
    display_name = "GCP Cloud Storage Buckets Modified Or Deleted"
    default_reference = "https://cloud.google.com/storage/docs/buckets"
    default_severity = Severity.LOW
    log_types = [LogType.GCP_AUDIT_LOG]
    id = "GCP.Cloud.Storage.Buckets.Modified.Or.Deleted-prototype"
    BUCKET_OPERATIONS = ["storage.buckets.delete", "storage.buckets.update"]

    def rule(self, event):
        return all(
            [
                event.deep_get("protoPayload", "serviceName", default="") == "storage.googleapis.com",
                event.deep_get("protoPayload", "methodName", default="") in self.BUCKET_OPERATIONS,
            ],
        )

    def title(self, event):
        actor = event.deep_get("protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        operation = event.deep_get("protoPayload", "methodName", default="<OPERATION_NOT_FOUND>")
        project = event.deep_get("resource", "labels", "project_id", default="<PROJECT_NOT_FOUND>")
        bucket = event.deep_get("resource", "labels", "bucket_name", default="<BUCKET_NOT_FOUND>")
        return f"GCP: [{actor}] performed a [{operation}] on bucket [{bucket}] in project [{project}]."

    tests = [
        RuleTest(
            name="other event",
            expected_result=False,
            log={
                "insertid": "ezyd47c12y",
                "logname": "projects/gcp-project1/logs/cloudaudit.googleapis.com%2Factivity",
                "p_any_ip_addresses": ["1.2.3.4"],
                "p_event_time": "2023-03-09 16:41:30.524",
                "p_log_type": "GCP.AuditLog",
                "p_parse_time": "2023-03-09 16:44:14.617",
                "p_row_id": "1234567909689348911",
                "p_source_id": "4fc88a5a-2d51-4279-9c4a-08fa7cc52566",
                "p_source_label": "gcplogsource",
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "test@company.io"},
                    "authorizationInfo": [
                        {
                            "granted": True,
                            "permission": "logging.sinks.update",
                            "resource": "projects/gcp-project1/sinks/log-sink",
                            "resourceAttributes": {
                                "name": "projects/gcp-project1/sinks/log-sink",
                                "service": "logging.googleapis.com",
                            },
                        },
                    ],
                    "methodName": "google.logging.v2.ConfigServiceV2.UpdateSink",
                    "request": {
                        "@type": "type.googleapis.com/google.logging.v2.UpdateSinkRequest",
                        "sink": {
                            "destination": "pubsub.googleapis.com/projects/gcp-project1/topics/gcp-topic1",
                            "exclusions": [{"filter": "protoPayload.serviceName = 'k8s.io", "name": "excludek8s"}],
                            "name": "log-sink",
                            "writerIdentity": "serviceAccount:p197946410614-915152@gcp-sa-logging.iam.gserviceaccount.com",
                        },
                        "sinkName": "projects/gcp-project1/sinks/log-sink",
                        "uniqueWriterIdentity": True,
                        "updateMask": "exclusions",
                    },
                    "requestMetadata": {
                        "callerIP": "1.2.3.4",
                        "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/110.0.0.0 Safari/537.36,gzip(gfe),gzip(gfe)",
                        "destinationAttributes": {},
                        "requestAttributes": {"auth": {}, "time": "2023-03-09T16:41:30.540045105Z"},
                    },
                    "resourceName": "projects/gcp-project1/sinks/log-sink",
                    "serviceName": "logging.googleapis.com",
                    "status": {},
                },
                "receivetimestamp": "2023-03-09 16:41:32.21",
                "resource": {
                    "labels": {"destination": "", "name": "log-sink", "project_id": "gcp-project1"},
                    "type": "logging_sink",
                },
                "severity": "NOTICE",
                "timestamp": "2023-03-09 16:41:30.524",
            },
        ),
        RuleTest(
            name="bucket update",
            expected_result=True,
            log={
                "insertId": "asdf1234asdfg",
                "logName": "projects/gcp-project1/logs/cloudaudit.googleapis.com%2Factivity",
                "p_any_ip_addresses": ["1.2.3.4"],
                "p_event_time": "2023-03-09 10:05:23.603",
                "p_log_type": "GCP.AuditLog",
                "p_parse_time": "2023-03-09 10:07:14.731",
                "p_row_id": "7ad218d42253b7e6f78cc0ed1635",
                "p_source_id": "4fc88a5a-2d51-4279-9c4a-08fa7cc52566",
                "p_source_label": "gcplogsource",
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "user@company.io"},
                    "authorizationInfo": [
                        {
                            "granted": True,
                            "permission": "storage.buckets.update",
                            "resource": "projects/_/buckets/my-bucket",
                            "resourceAttributes": {},
                        },
                    ],
                    "methodName": "storage.buckets.update",
                    "requestMetadata": {
                        "callerIP": "1.2.3.4",
                        "callerSuppliedUserAgent": "apitools Python/3.9.11 gsutil/5.11 (darwin) analytics/enabled interactive/True command/notification google-cloud-sdk/394.0.0,gzip(gfe)",
                        "destinationAttributes": {},
                        "requestAttributes": {"auth": {}, "time": "2023-03-09T10:05:23.610372568Z"},
                    },
                    "resourceName": "projects/_/buckets/my-bucket",
                    "serviceName": "storage.googleapis.com",
                    "status": {},
                },
                "receiveTimestamp": "2023-03-09 10:05:25.146",
                "resource": {
                    "labels": {"bucket_name": "my-bucket", "location": "us-east1", "project_id": "gcp-project1"},
                    "type": "gcs_bucket",
                },
                "severity": "NOTICE",
                "timestamp": "2023-03-09 10:05:23.603",
            },
        ),
    ]



"""
Helpers for Crowdstrike Event Streams detections.
"""

from pypanther.helpers.base import key_value_list_to_dict


def cs_alert_context(event):
    return audit_keys_dict(event)


def audit_keys_dict(event):
    return key_value_list_to_dict(
        event.deep_get("event", "AuditKeyValues", default=[]),
        "Key",
        "ValueString",
    )


def str_to_list(liststr: str) -> list[str]:
    """
    Several crowdstrike values are returned as a list like "[x y z]". This function convetrs
    such entries to Python list of strings, like: ["x", "y", "z"].
    """
    # Return empty list for empty string
    if not liststr:
        return []
    # Validate
    if liststr[0] != "[" or liststr[-1] != "]":
        raise ValueError(f"Invalid list string: {liststr}")
    return [x.strip() for x in liststr[1:-1].split(" ")]
